!```````````````````````````````````````````````````````````````````
module PRAMS_vtab_mod

!+ For Mars

use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN

implicit none

character(len=MAX_FILE_PATH_LEN) :: vtable_fn


contains


!```````````````````````````````````````````````````````````````````
subroutine vtable_setup(vtab_fn)

!+ Initialize variable tables (for Mars)

use univ_kind_defs_mod, only: f4
use univ_string_mod, only: deblank
use univ_exception_mod, only: exception, FATAL
use univ_IO_mem_status_mod, only: for_allocation
use univ_std_fileIO_mod, only: find_file_IOunit
use main_memory_mod, only: naddsc, n_grids, tnudcent, tnudtop,  &
     initialization_type, n_vars, gd
use main_memory_bspec_mod, only: dust_nb, H2Oi_nb, CO2i_nb, H2Ov_tog,  &
     H2Oi_tog, dust_tog, CO2i_tog
use alt_namelist_mod, only: altn_maxlen_line, tokenizer_flex,  &
     token_list_destroy, token_list
use vtables_mod

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: vtab_fn

!== Local declarations:

integer, parameter ::  maxtab = 20

character(len=altn_maxlen_line) :: in_line, line_format, vversion
character(len=8) :: cname,cnamex,cvar,ccond1,ccond2,cdimen
character(len=6) :: cversion
character(len=8), dimension(maxtab) :: ctable
character(len=1), dimension(1) :: toksep 

integer :: io_unit,n,igd,err,ntab,nlines  &
     ,nsclrp,nsclrt,naersclp,naersclt,loop,ivvv,iv,icond,nt,nv,ivflg,nvv  &
     ,rcd_iv,q1,q2,q3,q4,naersfi,naersff,naersaj,naersag,naersrs
logical :: we_will_use_this_var, version_is_OK

type(token_list) :: toklist

!=== Executable statements:

toksep(1) = ':'   ! Character that separates tokens

write(line_format,'(a2,i3.3,a1)') '(a', altn_maxlen_line, ')'

! Specify the basic VTABLE version that is required for this model version;
!    run will abort if it does not match that in the VTABLE file:

cversion='0002'

io_unit = find_file_IOunit()
open(io_unit,file=vtab_fn(1:len_trim(vtab_fn)),status='old') ! Open VTABLE file

! Determine how many records the file has:

n = 0
nv = 0
count: do while(.TRUE.)   ! Loop as many times as it takes
   n = n+1  ! Increment line count

   read(io_unit,line_format,iostat=err) in_line  ! Attempt to read another line
   if(err /= 0) then   ! If end of file
      exit count
   endif

   if(in_line(1:1) /= '#' .and. in_line(1:1) /= ' ' .and. & ! Only process  
        in_line(1:1) /= char(0)) then                       ! a valid record
      nv = nv+1
   endif

enddo count
nlines = n-1
! Number of VTABLE vars plus any added SCLP/SCLT vars *and* aerosol vars:
n_vars = nv + (naddsc-1)*2 + (1*7)*dust_nb + (2*7)*H2Oi_nb + (3*7)*CO2i_nb

rewind io_unit

call vtable_mem(for_allocation)    ! Allocate memory for variable tables

! Check first for the VERSION keyword; stop run if mismatch occurs:

version_is_OK = .FALSE.
do n=1,nlines
   read(io_unit,line_format) in_line

   if(in_line(1:8) == '#VERSION') then
      version_is_OK = .TRUE.    ! Preliminary "guess"

      call tokenizer_flex(in_line, toklist, toksep)
      if(toklist%n_tokens < 1) then
         call exception('vtable_setup', FATAL,  &
              'Unable to parse tokens from the following:'//trim(in_line))
      endif

      vversion = deblank(toklist%tokens(2))
      if(vversion(1:len_trim(vversion)) /= cversion(1:len_trim(cversion))) then
         version_is_OK = .FALSE.
      endif
   endif
enddo

if(.NOT.version_is_OK) then
   write (*,'(/a,/4a)') 'ERROR: VTABLE version mismatch.',  &
        'version in file = ', vversion(1:len_trim(vversion)),  &
        ', but PRAMS code needs version = ', cversion(1:len_trim(cversion))
   call exception('vtable_setup', FATAL)
endif

do igd=1,n_grids    ! Loop over all grids

   nv3d(igd) = 0
   nv2d(igd) = 0
   n_state_3D(igd) = 0
   n_state_2D(igd) = 0
   nv3ds(igd) = 0
   nv2ds(igd) = 0
   n_state_3Ds(igd) = 0
   n_state_2Ds(igd) = 0
   nv3dm(igd) = 0
   n_state_3Dm(igd) = 0
   nsclr(igd) = 0
   nsclrp = 0
   nsclrt = 0
   naersclr(igd) = 0
   naersclp = 0
   naersclt = 0
   naersfi = 0
   naersff = 0
   naersaj = 0
   naersag = 0
   naersrs = 0

   nmp31(igd) = 0
   nmp32(igd) = 0
   nmp33(igd) = 0
   nmp3i(igd) = 0
   nmp21(igd) = 0
   nmp22(igd) = 0
   nmp23(igd) = 0
   nmp2i(igd) = 0
   nmps31(igd) = 0
   nmps21(igd) = 0
   nmps32(igd) = 0
   nmps22(igd) = 0
   nmps33(igd) = 0
   nmps23(igd) = 0
   nmps3i(igd) = 0
   nmps2i(igd) = 0

   rewind io_unit      ! "Rewind" file to its beginning

   rcd_iv = 0
   do n=1,nlines   ! Loop over all records in file
  
      in_line = ';'    ! (Re)initialize string storage
      read (io_unit,line_format) in_line    ! Read in a record

      if(in_line(1:1) /= '#' .and. in_line(1:1) /= ' ' .and. & ! Only process  
           in_line(1:1) /= char(0)) then                       ! a valid record
                                                                                
         call tokenizer_flex(in_line, toklist, toksep)
         if(toklist%n_tokens < 1) then
            call exception('vtable_setup', FATAL,  &
                 '(tf02): unable to parse tokens from the following:'//trim(in_line))
         endif
         if(toklist%n_tokens < 6) then
            call exception('vtable_setup', FATAL,  &
                 '(tf02): not enough tokens in the following:'//trim(in_line))
         endif

         cname = toklist%tokens(1)
         cvar = toklist%tokens(2)
         cdimen = toklist%tokens(3)
         ccond1 = toklist%tokens(4)
         ccond2 = toklist%tokens(5)
         ntab = toklist%n_tokens-5
         ctable(1:ntab) = toklist%tokens(6:toklist%n_tokens)
         
         read (cvar,*) ivvv   ! `ivvv` now holds variable ID#
         if(ivvv > 0) then
            rcd_iv = ivvv    ! Record current "real" variable number
         endif

         loop = 1   ! Normally only loop once
         if(cname == 'SCLP' .or. cname == 'SCLT') then
            loop = max(1,naddsc)    ! Loop over all added scalars
            cnamex = cname
         elseif(cname == 'DUSTNC' .or. cname == 'DUSTNT' .or.  &
              cname == 'DUSTRS' .or. cname == 'DUSTFI' .or.  &
              cname == 'DUSTFF' .or. cname == 'DUSTAJ' .or.  &
              cname == 'DUSTAG') then
            loop = max(1,dust_nb)   ! Loop over all dust bins
            cnamex = cname
            if(cname(6:6) == 'C' .or. cname(6:6) == 'R') then           !
               unvar_bndidx(1,1) = rcd_iv+1      ! 
            elseif(cname(6:6) == 'T') then       ! Record unknowns' starting
               unvar_bndidx(1,2) = rcd_iv+1      !  indexes (per var group)
            elseif(cname(6:6) == 'I') then       !
               unvar_bndidx(1,3) = rcd_iv+1      !
            elseif(cname(6:6) == 'F') then       !
               unvar_bndidx(1,4) = rcd_iv+1      !
            elseif(cname(6:6) == 'J') then       !
               unvar_bndidx(1,5) = rcd_iv+1      !
            elseif(cname(6:6) == 'G') then       !
               unvar_bndidx(1,6) = rcd_iv+1      !
            elseif(cname(6:6) == 'S') then       !
               unvar_bndidx(1,7) = rcd_iv+1      !
            endif                                !
         elseif(cname == 'H2OINC' .or. cname == 'H2OINT' .or.  &
              cname == 'H2OIFI' .or. cname == 'H2OIFF' .or.  &
              cname == 'H2OIAJ' .or. cname == 'H2OIAG' .or.  &
              cname == 'H2OIRS' .or.  &
              cname == 'H2ODCR' .or. cname == 'H2ODCT' .or.  &
              cname == 'H2ODFI' .or. cname == 'H2ODFF' .or.  &
              cname == 'H2ODAJ' .or. cname == 'H2ODAG' .or.  &
              cname == 'H2ODRS') then
            loop = max(1,H2Oi_nb)   ! Loop over all H2O bins
            cnamex = cname
         elseif(cname == 'CO2INC' .or. cname == 'CO2INT' .or.  &
              cname == 'CO2IFI' .or. cname == 'CO2IFF' .or.  &
              cname == 'CO2IAJ' .or. cname == 'CO2IAG' .or.  &
              cname == 'CO2IRS' .or.  &
              cname == 'CO2DCR' .or. cname == 'CO2DCT' .or.  &
              cname == 'CO2DFI' .or. cname == 'CO2DFF' .or.  &
              cname == 'CO2DAJ' .or. cname == 'CO2DAG' .or.  &
              cname == 'CO2DRS' .or.  &
              cname == 'CO2WCR' .or. cname == 'CO2WCT' .or.  &
              cname == 'CO2WFI' .or. cname == 'CO2WFF' .or.  &
              cname == 'CO2WAJ' .or. cname == 'CO2WAG' .or.  &
              cname == 'CO2WRS') then
            loop = max(1,CO2i_nb)   ! Loop over all CO2 bins
            cnamex = cname
         else
            cnamex = ';'
         endif
         
         do iv=1,loop

            if(cnamex == 'SCLP') then
               write (cname,'(a4,i3.3)') 'SCLR',iv ! now called e.g.,'SCLR001'
               write (cvar,'(i3)') rcd_iv+iv    ! Increment var ID#
            elseif(cnamex == 'SCLT') then
               write (cname,'(a4,i3.3)') 'SCLT',iv ! now called e.g.,'SCLT001'
               write (cvar,'(i3)') rcd_iv+iv   ! Increment var ID#
            elseif(cnamex /= ';') then   ! Deal with aerosol variables
               ! now known as e.g.,'DUSTCR01':
               write (cname,'(a6,i2.2)') cnamex(1:6),iv 
               write (cvar,'(i3)') rcd_iv+iv   ! Increment var ID#
            endif
            
            read (ccond1,*) icond   ! Condition number
            
            we_will_use_this_var = .FALSE.
            
            if(icond == 0) then  ! No condition for use
               we_will_use_this_var = .TRUE.

            elseif(icond == 1) then   ! Only if water vapor desired
               if(H2Ov_tog > 0) then
                  we_will_use_this_var = .TRUE.
               endif

            elseif(icond == 2) then  ! SGS TKE field needed
               if (gd(igd)%idiffk == 1 .or. gd(igd)%idiffk == 4 .or.  &
                    gd(igd)%idiffk == 5 .or. gd(igd)%idiffk == 7 .or.  &
                    gd(igd)%idiffk == 9) then
                  we_will_use_this_var = .TRUE.
               end if

            else if (icond == 3) then  ! Only if a variable init run, etc.
               if (initialization_type == 2) then
                  we_will_use_this_var = .not.( igd > 1 .and.  &
                       (tnudcent+tnudtop) < 0.001_f4 )
               end if

            elseif(icond == 4) then  ! Only if additional scalars requested
               if(naddsc >= 1) then
                  we_will_use_this_var = .TRUE.
               endif

            elseif(icond == 5) then   ! Only if dust desired
               if(dust_tog > 0) then
                  we_will_use_this_var = .TRUE.
               endif

            elseif(icond == 6) then   ! Only if water ice nucleation desired
               if(H2Oi_tog > 0) then
                  we_will_use_this_var = .TRUE.
               endif

            elseif(icond == 7) then   ! Only if CO2 ice nucleation desired
               if(CO2i_tog > 0) then
                  we_will_use_this_var = .TRUE.
               endif

            elseif(icond == 8) then   ! Only if CO2 ice AND H2O_ice nuc desired
               if(CO2i_tog > 0 .and. H2Oi_tog > 0) then
                  we_will_use_this_var = .TRUE.
               endif

            endif

            if(we_will_use_this_var) then

               do nt=1,ntab

                  ! Grid variable table:
                  if(ctable(nt) == 'vard' .and. cdimen == '3') then
                     call vtabfill(cvar,nv3d(igd),ivtab3d(:,igd)  &
                          ,cname,ivchr3d(:,igd))
                     ivopt3d(nv3d(igd),igd) = ccond2
                  elseif(ctable(nt) == 'vari' .and. cdimen == '3') then
                     call vtabfill(cvar,nv3dm(igd),ivtab3dm(:,igd)  &
                          ,cname,ivchr3dm(:,igd))
                     ivopt3dm(nv3dm(igd),igd) = ccond2
                  elseif(ctable(nt) == 'vard' .and. cdimen == '2') then
                     call vtabfill(cvar,nv2d(igd),ivtab2d(:,igd)  &
                          ,cname,ivchr2d(:,igd))
                     ivopt2d(nv2d(igd),igd) = ccond2
                  elseif(ctable(nt) == 'vard' .and. cdimen == 's3') then
                     call vtabfill(cvar,nv3ds(igd),ivtab3ds(:,igd)  &
                          ,cname,ivchr3ds(:,igd))
                     ivopt3ds(nv3ds(igd),igd) = ccond2
                  elseif(ctable(nt) == 'vard' .and. cdimen == 's2') then
                     call vtabfill(cvar,nv2ds(igd),ivtab2ds(:,igd)  &
                          ,cname,ivchr2ds(:,igd))
                     ivopt2ds(nv2ds(igd),igd) = ccond2
                  endif
                  
                  ! State file variable table:
                  if(ctable(nt) == 'stat' .and. cdimen == '3') then
                     call vtabfill(cvar, n_state_3D(igd),  &
                          i_state_tab3D(:,igd), cname, i_state_chr3D(:,igd))
                  elseif(ctable(nt) == 'stat' .and. cdimen == '2') then
                     call vtabfill(cvar, n_state_2D(igd),  &
                          i_state_tab2D(:,igd), cname, i_state_chr2D(:,igd))
                  elseif(ctable(nt) == 'stat' .and. cdimen == 's3') then
                     call vtabfill(cvar, n_state_3Ds(igd),  &
                          i_state_tab3Ds(:,igd), cname, i_state_chr3Ds(:,igd))
                  elseif(ctable(nt) == 'stat' .and. cdimen == 's2') then
                     call vtabfill(cvar, n_state_2Ds(igd),  &
                          i_state_tab2Ds(:,igd), cname, i_state_chr2Ds(:,igd))
                  endif

                  ! Scalar variable table:
                  if(ctable(nt) == 'sclp' .and. cdimen == '3') then
                     call vtabfill(cvar,nsclrp,isclrp(:,igd)  &
                          ,cname,ischrp(:,igd))                
                  elseif(ctable(nt) == 'sclt' .and. cdimen == '3') then
                     call vtabfill(cvar,nsclrt,isclrt(:,igd)  &
                          ,cname,ischrt(:,igd))
                  endif

                  ! Aerosol scalar variable tables:
                  if(ctable(nt) == 'aer_sclp' .and. cdimen == '3') then
                     call vtabfill(cvar,naersclp,iaersclp(:,igd)  &
                          ,cname,iaerschrp(:,igd))                
                  elseif(ctable(nt) == 'aer_sclt' .and. cdimen == '3') then
                     call vtabfill(cvar,naersclt,iaersclt(:,igd)  &
                          ,cname,iaerschrt(:,igd))
                  elseif(ctable(nt) == 'aer_sfi' .and. cdimen == '2') then
                     call vtabfill(cvar,naersfi,iaersfi(:,igd)  &
                          ,cname,iaerschrfi(:,igd))
                  elseif(ctable(nt) == 'aer_sff' .and. cdimen == '2') then
                     call vtabfill(cvar,naersff,iaersff(:,igd)  &
                          ,cname,iaerschrff(:,igd))
                  elseif(ctable(nt) == 'aer_saj' .and. cdimen == '2') then
                     call vtabfill(cvar,naersaj,iaersaj(:,igd)  &
                          ,cname,iaerschraj(:,igd))
                  elseif(ctable(nt) == 'aer_sag' .and. cdimen == '2') then
                     call vtabfill(cvar,naersag,iaersag(:,igd)  &
                          ,cname,iaerschrag(:,igd))
                  elseif(ctable(nt) == 'aer_srs' .and. cdimen == '2') then
                     call vtabfill(cvar,naersrs,iaersrs(:,igd)  &
                          ,cname,iaerschrrs(:,igd))
                  endif

                  ! Parallel BC node-to-node (LONG dt) variable table:
                  if(ctable(nt) == 'mpt1' .and. cdimen == '3') then
                     call vtabfill(cvar,nmp31(igd),mp3tab1(:,igd)  &
                          ,cname,mp3chr1(:,igd))
                  elseif(ctable(nt) == 'mpt1' .and. cdimen == '2') then
                     call vtabfill(cvar,nmp21(igd),mp2tab1(:,igd)  &
                          ,cname,mp2chr1(:,igd))
                  elseif(ctable(nt) == 'mpt1' .and. cdimen == 's3') then
                     call vtabfill(cvar,nmps31(igd),mp3stab1(:,igd)  &
                          ,cname,mp3schr1(:,igd))
                  elseif(ctable(nt) == 'mpt1' .and. cdimen == 's2') then
                     call vtabfill(cvar,nmps21(igd),mp2stab1(:,igd)  &
                          ,cname,mp2schr1(:,igd))
                  endif

                  ! Parallel BC node-to-node (SHORT dt) variable table:
                  if(ctable(nt) == 'mpt2' .and. cdimen == '3') then
                     call vtabfill(cvar,nmp32(igd),mp3tab2(:,igd)  &
                          ,cname,mp3chr2(:,igd))
                  elseif(ctable(nt) == 'mpt2' .and. cdimen == '2') then
                     call vtabfill(cvar,nmp22(igd),mp2tab2(:,igd)  &
                          ,cname,mp2chr2(:,igd))
                  elseif(ctable(nt) == 'mpt2' .and. cdimen == 's3') then
                     call vtabfill(cvar,nmps32(igd),mp3stab2(:,igd)  &
                          ,cname,mp3schr2(:,igd))
                  elseif(ctable(nt) == 'mpt2' .and. cdimen == 's2') then
                     call vtabfill(cvar,nmps22(igd),mp2stab2(:,igd)  &
                          ,cname,mp2schr2(:,igd))
                  endif
                  
                  ! Full subdomain node-to-master (for output) variable table:
                  if(ctable(nt) == 'mpt3' .and. cdimen == '3') then
                     call vtabfill(cvar,nmp33(igd),mp3tab3(:,igd)  &
                          ,cname,mp3chr3(:,igd))
                  elseif(ctable(nt) == 'mpt3' .and. cdimen == '2') then
                     call vtabfill(cvar,nmp23(igd),mp2tab3(:,igd)  &
                          ,cname,mp2chr3(:,igd))
                  elseif(ctable(nt) == 'mpt3' .and. cdimen == 's3') then
                     call vtabfill(cvar,nmps33(igd),mp3stab3(:,igd)  &
                          ,cname,mp3schr3(:,igd))
                  elseif(ctable(nt) == 'mpt3' .and. cdimen == 's2') then
                     call vtabfill(cvar,nmps23(igd),mp2stab3(:,igd)  &
                          ,cname,mp2schr3(:,igd))
                  endif

                  ! Full subdomain master-to-node (init) variable table:
                  if(ctable(nt) == 'mpti' .and. cdimen == '3') then
                     call vtabfill(cvar,nmp3i(igd),mp3tabi(:,igd)  &
                          ,cname,mp3chri(:,igd))
                  elseif(ctable(nt) == 'mpti' .and. cdimen == '2') then
                     call vtabfill(cvar,nmp2i(igd),mp2tabi(:,igd)  &
                          ,cname,mp2chri(:,igd))
                  elseif(ctable(nt) == 'mpti' .and. cdimen == 's3') then
                     call vtabfill(cvar,nmps3i(igd),mp3stabi(:,igd)  &
                          ,cname,mp3schri(:,igd))
                  elseif(ctable(nt) == 'mpti' .and. cdimen == 's2') then
                     call vtabfill(cvar,nmps2i(igd),mp2stabi(:,igd)  &
                          ,cname,mp2schri(:,igd))
                  endif

               enddo

            endif
         enddo

         if(ivvv <= 0) then    ! Only do if an "unknown" variable idx 
            rcd_iv = rcd_iv+loop   ! Update variable_index counter
         endif

      endif
   enddo

   if(nsclrp == nsclrt) then
      nsclr(igd) = nsclrp
   else
      call exception('vtable_setup', FATAL,  &
           'scalar variable count (nsclrp /= nsclrt)')
   endif

   if(naersclp == naersclt) then
      naersclr(igd) = naersclp
   else
      call exception('vtable_setup', FATAL,  &
           'aerosol scalar variable count (naersclp /= naersclt)')
   endif
   
enddo

close(io_unit)  ! Close VTABLE file

! The 3D grid-independent variable table has been filled for each grid.
!   Combine this into a single table:

nv3dmt = 0
do igd=1,n_grids
   do nv=1,nv3dm(igd)

      ivflg = 0
      do nvv=1,nv3dmt
         if(ivtab3dm(nv,igd) == ivtab3m(nvv)) then
            ivflg = 1
         endif
      enddo
      if(ivflg == 0) then
         nv3dmt = nv3dmt+1
         ivtab3m(nv3dmt) = ivtab3dm(nv,igd)
         ivopt3m(nv3dmt) = ivopt3dm(nv,igd)
      endif

   enddo
enddo

! Finish recording variable index boundaries for "unknown" variables:

q1 = max(1,dust_nb)
q2 = max(1,H2Oi_nb)
q3 = max(1,CO2i_nb)
q4 = max(1,naddsc)

! 3D grid-dependent unknowns' var_idx bounds ('NC','CR'):
unvar_bndidx(2,1) = unvar_bndidx(1,1)+q1    
unvar_bndidx(3,1) = unvar_bndidx(2,1)+q2
unvar_bndidx(4,1) = unvar_bndidx(3,1)+q2
unvar_bndidx(5,1) = unvar_bndidx(4,1)+q3
unvar_bndidx(6,1) = unvar_bndidx(5,1)+q3
unvar_bndidx(7,1) = unvar_bndidx(6,1)+q3
unvar_bndidx(8,1) = unvar_bndidx(7,1)+q4     ! SCLP

! 3D grid-independent unknowns' var_idx bounds  ('NT','CT'):
unvar_bndidx(2,2) = unvar_bndidx(1,2)+q1
unvar_bndidx(3,2) = unvar_bndidx(2,2)+q2
unvar_bndidx(4,2) = unvar_bndidx(3,2)+q2
unvar_bndidx(5,2) = unvar_bndidx(4,2)+q3
unvar_bndidx(6,2) = unvar_bndidx(5,2)+q3
unvar_bndidx(7,2) = unvar_bndidx(6,2)+q3
unvar_bndidx(8,2) = unvar_bndidx(7,2)+q4     ! SCLT

! 2D grid-dependent unknowns' var_idx bounds ('FI','FF','AJ','AG','RS'):
unvar_bndidx(2,3) = unvar_bndidx(1,3)+q1    !
unvar_bndidx(3,3) = unvar_bndidx(2,3)+q2    !
unvar_bndidx(4,3) = unvar_bndidx(3,3)+q2    ! 'FI'
unvar_bndidx(5,3) = unvar_bndidx(4,3)+q3    !
unvar_bndidx(6,3) = unvar_bndidx(5,3)+q3    !
unvar_bndidx(7,3) = unvar_bndidx(6,3)+q3    !

unvar_bndidx(2,4) = unvar_bndidx(1,4)+q1    !
unvar_bndidx(3,4) = unvar_bndidx(2,4)+q2    !
unvar_bndidx(4,4) = unvar_bndidx(3,4)+q2    ! 'FF'
unvar_bndidx(5,4) = unvar_bndidx(4,4)+q3    !
unvar_bndidx(6,4) = unvar_bndidx(5,4)+q3    !
unvar_bndidx(7,4) = unvar_bndidx(6,4)+q3    !

unvar_bndidx(2,5) = unvar_bndidx(1,5)+q1    !
unvar_bndidx(3,5) = unvar_bndidx(2,5)+q2    !
unvar_bndidx(4,5) = unvar_bndidx(3,5)+q2    ! 'AJ'
unvar_bndidx(5,5) = unvar_bndidx(4,5)+q3    !
unvar_bndidx(6,5) = unvar_bndidx(5,5)+q3    !
unvar_bndidx(7,5) = unvar_bndidx(6,5)+q3    !

unvar_bndidx(2,6) = unvar_bndidx(1,6)+q1    !
unvar_bndidx(3,6) = unvar_bndidx(2,6)+q2    !
unvar_bndidx(4,6) = unvar_bndidx(3,6)+q2    ! 'AG'
unvar_bndidx(5,6) = unvar_bndidx(4,6)+q3    !
unvar_bndidx(6,6) = unvar_bndidx(5,6)+q3    !
unvar_bndidx(7,6) = unvar_bndidx(6,6)+q3    !

unvar_bndidx(2,7) = unvar_bndidx(1,7)+q1    !
unvar_bndidx(3,7) = unvar_bndidx(2,7)+q2    !
unvar_bndidx(4,7) = unvar_bndidx(3,7)+q2    ! 'RS'
unvar_bndidx(5,7) = unvar_bndidx(4,7)+q3    !
unvar_bndidx(6,7) = unvar_bndidx(5,7)+q3    !
unvar_bndidx(7,7) = unvar_bndidx(6,7)+q3    !

call token_list_destroy(toklist)

end subroutine vtable_setup


!```````````````````````````````````````````````````````````````````
subroutine vtabfill(cvar,ntab,itab,cname,ctable)

!+ Fill variable table values for a single variable

implicit none

!=== Argument declarations:

character(len=*), intent(IN) :: cvar,cname

integer, intent(INOUT) :: ntab
integer, dimension(:), intent(OUT) :: itab
character(len=*), dimension(:), intent(OUT) :: ctable

!== Local declarations:

integer :: ivar

!=== Executable statements:

read (cvar,*) ivar
ntab = ntab+1
itab(ntab) = ivar
ctable(ntab) = cname

end subroutine vtabfill


end module PRAMS_vtab_mod
