!```````````````````````````````````````````````````````````````````
module main_memory_bspec_mod

!+ For Mars

use univ_kind_defs_mod, only: f4, f8
use manage_NetCDF_classic_mod, only: mNc_file_metadata_obj
use namelist_maxdims_mod, only: nml__strtoklen_max, nml_fn_strtoklen_max
use main_memory_mod, only: trace_coord_obj

implicit none

!== Namelist parameters, sorted by namelist block:

! &model_options_bspec :
integer :: RTc_tog, lonrad, ishadtog, lfmcut, nrdlevt, RT_flux_trace_np,  &
     bdst_tog, islastog, H2Ov_tog, H2Ov_ivr, dust_tog, dust_nb, dust_ivr,  &
     H2Oi_tog, H2Oi_nb, CO2i_tog, CO2i_nb, CO2_sfci, nslcon, dust_op_case
real(f4) :: tautot, conrnu, bdst_imr, bdst_igd, bdst_icn, bdst_irt, bdst_iht,  &
     bdst_itt, dust_rmn, dust_mrt, dust_jef, dust_jth, dust_igd, dust_imr,  &
     dust_inn, dust_icn, dust_irt, dust_iht, dust_itt, H2Oi_rmn, H2Oi_mrt,  &
     CO2i_rmn, CO2i_mrt, pctlcon, sfci_iLs, Ls_of_periapse, dust_ctau_refp,  &
     orbital_eccentricity, axial_obliquity, AU_to_m_knob
real(f8) :: dust_rtt, H2Oi_rtt, CO2i_rtt, H2Ov_beg, dust_beg, dust_jsr,  &
     dust_jbg, H2Oi_beg, CO2i_beg
logical :: dust_rta, H2Oi_rta, CO2i_rta, use_gas_opacities
character(len=nml__strtoklen_max) :: dust_rtt_repstr, H2Oi_rtt_repstr,  &
     CO2i_rtt_repstr
character(len=nml__strtoklen_max), dimension(2) :: RT_flux_trace_ti_repstr
real(f8), dimension(2) :: RT_flux_trace_ti
real(f4), allocatable, dimension(:) :: slz, stgoff, slmstr
type(trace_coord_obj), allocatable, dimension(:) :: RT_flux_trace_pt

! &initial_sounding :
integer :: nsndp, ipsflg, itsflg, irtsflg, iusflg
logical :: use_ext_sounding_file
character(len=nml_fn_strtoklen_max) :: input_sounding_fn
real(f4), allocatable, dimension(:) :: hsnd, psnd, tsnd, rtsnd,  &
     usnd, vsnd

!== Other non-parallel-computing variables and parameters:

! NONE, currently

!=== Prognostic fields:

! NONE, currently

!=== Diagnostic fields:

! NONE, currently

!== Non-namelist grid-INDEPENDENT variables and control parameters:

type(mNc_file_metadata_obj), allocatable, dimension(:) ::  & ! {RT_flux_trace_np}
     RT_flux_trace_outp_f_obj
real(f4), allocatable, dimension(:) :: thdsnd   ! {nsndp}
integer, allocatable, dimension(:) :: n_RT_flux_snapshots

! 1D RT_flux_trace arrays {RT_flux_trace_np}:
logical, allocatable, dimension(:) :: ip_mask

! 3D RT_flux_trace arrays {z,iw,RT_flux_trace_np}:
real(f4), allocatable, dimension(:,:,:) :: fluxout_up, fluxout_dn

! 2D RT_flux_trace array {z,RT_flux_trace_np}:
real(f4), allocatable, dimension(:,:) :: dT_dt_out

!== Parallel-computing grid-INDEPENDENT variables and control parameters:

!== Grid-DEPENDENT variables and control parameters:

type grid_bspec_obj_type
   !== Grid-dependent namelist parameters, sorted by namelist block:

   ! NONE, currently

   !== Non-namelist grid-dependent variables and control parameters:

   ! Reference state arrays {maxnzp}:
   real(f4), pointer, dimension(:) :: rt01d => NULL()

   ! Normal 3D atmospheric arrays {z,x,y}:
   real(f4), pointer, dimension(:) :: rv_backup => NULL()

   ! Aerosol 3D atmospheric arrays {z,x,y,aerosol_idx}:
   real(f4), pointer, dimension(:) :: dustnc_backup => NULL(),  &
        H2Oinc_backup => NULL(), H2Odcr_backup => NULL(),  &
        CO2inc_backup => NULL(), CO2dcr_backup => NULL(),  &
        CO2wcr_backup => NULL()

   ! Normal 2D "atmospheric" arrays {x,y}:
   real(f4), pointer, dimension(:) :: qfz_backup => NULL()

   ! Aerosol 2D "atmospheric" arrays {x,y,aerosol_idx}:
   real(f4), pointer, dimension(:) :: dustfi_backup => NULL(),  &
        H2Oifi_backup => NULL(), H2Odfi_backup => NULL(),  &
        CO2ifi_backup => NULL(), CO2dfi_backup => NULL(),  &
        CO2wfi_backup => NULL(), dustff_backup => NULL(),  &
        H2Oiff_backup => NULL(), H2Odff_backup => NULL(),  &
        CO2iff_backup => NULL(), CO2dff_backup => NULL(),  &
        CO2wff_backup => NULL(), dustaj_backup => NULL(),  &
        H2Oiaj_backup => NULL(), H2Odaj_backup => NULL(),  &
        CO2iaj_backup => NULL(), CO2daj_backup => NULL(),  &
        CO2waj_backup => NULL(), dustag_backup => NULL(),  &
        H2Oiag_backup => NULL(), H2Odag_backup => NULL(),  &
        CO2iag_backup => NULL(), CO2dag_backup => NULL(),  &
        CO2wag_backup => NULL(), dustrs_backup => NULL(),  &
        H2Oirs_backup => NULL(), H2Odrs_backup => NULL(),  &
        CO2irs_backup => NULL(), CO2drs_backup => NULL(),  &
        CO2wrs_backup => NULL()

   ! Normal 3D sub/super-surface arrays {z,x,y,patch}:
   real(f4), pointer, dimension(:) :: SBPSH2Oa_backup => NULL()

   ! Normal 2D sub/super-surface arrays {x,y,patch}:
   real(f4), pointer, dimension(:) :: sfcH2Omr_backup => NULL()

   !== Parallel-computing grid-dependent variables and control parameters:
   
   !------------------ used by: root PE *and* leaf PE ------------------------


   !------------------ root_PE-only variables --------------------------------


   !------------------ leaf_PE-only variables --------------------------------
 

end type grid_bspec_obj_type

type(grid_bspec_obj_type), allocatable, target, dimension(:) :: gd_bd
type(grid_bspec_obj_type), pointer :: c_gd_bd => NULL()


contains


!```````````````````````````````````````````````````````````````````
subroutine manage_nml_var_memory_bspec(dim_n_grids, dim_nzg, dim_nsndp,  &
     dim_RT_flux_trace_np)

!+ Allocates or deallocates namelist-related arrays with various dimensions

use univ_IO_mem_status_mod, only: illegal_alloc_stop
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use main_memory_mod, only: nzg, gd, n_grids

implicit none

!=== Argument declarations:

logical, optional, intent(IN) :: dim_n_grids, dim_nzg, dim_nsndp,  &
     dim_RT_flux_trace_np

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'manage_nml_var_memory_bspec'

integer :: a_err, igd

!=== Executable statements:

if (present(dim_n_grids)) then
   if (dim_n_grids) then
      allocate(gd_bd(n_grids), stat=a_err)
   else
      deallocate(gd_bd, stat=a_err)
   end if
   if (a_err /= 0) call illegal_alloc_stop(dim_n_grids, procedure_name,  &
        'dim_n_grids')

   ! Nullify all component pointer elements of each grid object:
   if (dim_n_grids) then
      do igd=1,n_grids
         call manage_grid_object_bspec(gd_bd(igd), gd(igd),  &
              nullify_pointers=.TRUE.)
      end do
   end if
end if

if (present(dim_nzg)) then
   if (dim_nzg) then
      allocate(slz(nzg), stgoff(nzg), slmstr(nzg), stat=a_err)  
   else
      deallocate(slz, stgoff, slmstr, stat=a_err)
   end if
   if (a_err /= 0) call illegal_alloc_stop(dim_nzg, procedure_name, 'dim_nzg')
end if

if (present(dim_nsndp)) then
   if (dim_nsndp) then
      allocate(hsnd(nsndp), psnd(nsndp), tsnd(nsndp), rtsnd(nsndp),  &
           usnd(nsndp), vsnd(nsndp), thdsnd(nsndp), stat=a_err)
   else
      deallocate(hsnd, psnd, tsnd, rtsnd, usnd, vsnd, thdsnd, stat=a_err)
   end if
   if (a_err /= 0) call illegal_alloc_stop(dim_nsndp, procedure_name,  &
        'dim_nsndp')
end if

if (present(dim_RT_flux_trace_np)) then
   if (dim_RT_flux_trace_np) then
      allocate(RT_flux_trace_pt(RT_flux_trace_np),  &
           RT_flux_trace_outp_f_obj(RT_flux_trace_np),  &
           n_RT_flux_snapshots(RT_flux_trace_np), stat=a_err)
   else
      deallocate(RT_flux_trace_pt, RT_flux_trace_outp_f_obj,  &
           n_RT_flux_snapshots, stat=a_err)
   end if
   if (a_err /= 0) call illegal_alloc_stop(dim_RT_flux_trace_np,  &
        procedure_name, 'dim_RT_flux_trace_np')
end if

end subroutine manage_nml_var_memory_bspec


!```````````````````````````````````````````````````````````````````
subroutine manage_grid_object_bspec(o, o_n, nullify_pointers,  &
     recycle_pointers, allocate_pd_ind_pointers, allocate_pd_dep_pointers)

!+ Manages the component pointer elements of an object of type
!  'grid_bspec_obj_type' 

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_pointer_utils_mod, only: ptr_release
use main_memory_mod, only: grid_obj_type, dt_mode, maxnzp, my_rank,  &
     rootPE_rank, run_is_parallel, nzs, nzg, npatch

implicit none

!=== Argument declarations:

type(grid_bspec_obj_type), intent(INOUT) :: o  ! Object to be operated on
type(grid_obj_type), intent(INOUT) :: o_n  ! "Normal" grid object

! Nullify ALL pointers (no allocation check)?
logical, optional, intent(IN) :: nullify_pointers

! "Recycle" (deallocate if allocated, then nullify) ALL pointers?
logical, optional, intent(IN) :: recycle_pointers

! Allocate all pointers that are INDEPENDENT of the parallel decomposition?
logical, optional, intent(IN) :: allocate_pd_ind_pointers

! Allocate all pointers that are DEPENDENT on the parallel decomposition?
logical, optional, intent(IN) :: allocate_pd_dep_pointers

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'manage_grid_object_bspec'

integer :: e, dim_z, dim_o
logical :: np, rp, e_check, ai_pr, ad_pr, ai_p, ad_p, ai_pl, ad_pl, ai,  &
     ad, ad_tc

!=== Executable statements:

! * Parallel-decomposition DEPENDENT arrays change shape/size depending on
!   whether this code is executed by the root PE or a leaf PE.
! * In general, arrays allocated only by the root PE are pd-INDEPENDENT. 

! Initialize control parameters and process optional arguments:
np = .FALSE.; rp = .FALSE.; ai = .FALSE.; ad = .FALSE.
if (present(nullify_pointers)) np = nullify_pointers
if (present(recycle_pointers)) rp = recycle_pointers
if (present(allocate_pd_ind_pointers)) ai = allocate_pd_ind_pointers
if (present(allocate_pd_dep_pointers)) ad = allocate_pd_dep_pointers

! Is this a parallel run?
ai_p = ai .and. run_is_parallel
ad_p = ad .and. run_is_parallel

! Is this the root PE of a parallel run?
ai_pr = ai_p .and. my_rank == rootPE_rank
ad_pr = ad_p .and. my_rank == rootPE_rank

! Is this a leaf PE of a parallel run?
ai_pl = ai_p .and. my_rank /= rootPE_rank
ad_pl = ad_p .and. my_rank /= rootPE_rank

! Allocate timestep model state copy memory (serial root or leaf PE, only)?
ad_tc = ad .and. dt_mode < 0 .and. (.NOT.ad_p .or. ad_pl)

! Initially there are no errors:
e_check = .FALSE.; e = 0

!== {pd-INDEPENDENT} "Vertical" 1D arrays {maxnzp}:

!VARY: use correct size(s)?
if (ai) dim_z = maxnzp

if (np) nullify(o%rt01d); if (rp) call ptr_release(o%rt01d, e)
if (ai) allocate(o%rt01d(dim_z), stat=e)
e_check = e_check .or. (e /= 0)

if (e_check) call exception(procedure_name, FATAL,  &
     '{pd-INDEPENDENT} Vertical 1D array {maxnzp}')

!== {pd-DEPENDENT} State copy 3D atmospheric arrays {nxyzp}:

if (ad_tc) then
   if (my_rank == rootPE_rank) then
      dim_o = o_n%nxyzp
   else
      dim_o = o_n%mmzp*o_n%mmxp*o_n%mmyp
   end if
end if

if (np) nullify(o%rv_backup); if (rp) call ptr_release(o%rv_backup, e)
if (H2Ov_tog > 0) then
   if (ad_tc) allocate(o%rv_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (e_check) call exception(procedure_name, FATAL,  &
     '{pd-DEPENDENT} State copy 3D atmospheric arrays {nxyzp}')

!== {pd-DEPENDENT} State copy 3D aerosol arrays {nxyzp,*}:

if (ad_tc) then
   if (my_rank == rootPE_rank) then
      dim_o = o_n%nxyzp*max(1, dust_nb)
   else
      dim_o = o_n%mmzp*o_n%mmxp*o_n%mmyp*max(1, dust_nb)
   end if
end if

if (np) nullify(o%dustnc_backup); if (rp) call ptr_release(o%dustnc_backup, e)
if (dust_tog > 0) then
   if (ad_tc) allocate(o%dustnc_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (ad_tc) then
   if (my_rank == rootPE_rank) then
      dim_o = o_n%nxyzp*max(1, H2Oi_nb)
   else
      dim_o = o_n%mmzp*o_n%mmxp*o_n%mmyp*max(1, H2Oi_nb)
   end if
end if

if (np) nullify(o%H2Oinc_backup); if (rp) call ptr_release(o%H2Oinc_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Oinc_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%H2Odcr_backup); if (rp) call ptr_release(o%H2Odcr_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Odcr_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (ad_tc) then
   if (my_rank == rootPE_rank) then
      dim_o = o_n%nxyzp*max(1, CO2i_nb)
   else
      dim_o = o_n%mmzp*o_n%mmxp*o_n%mmyp*max(1, CO2i_nb)
   end if
end if

if (np) nullify(o%CO2inc_backup); if (rp) call ptr_release(o%CO2inc_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2inc_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2dcr_backup); if (rp) call ptr_release(o%CO2dcr_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2dcr_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2wcr_backup); if (rp) call ptr_release(o%CO2wcr_backup, e)
if (CO2i_tog > 0 .and. H2Oi_tog > 0) then
   if (ad_tc) allocate(o%CO2wcr_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (e_check) call exception(procedure_name, FATAL,  &
     '{pd-DEPENDENT} State copy 3D aerosol arrays {nxyzp,*}')

!== {pd-DEPENDENT} State copy 2D aerosol arrays {nxyp,*}:

if (ad_tc) then
   if (my_rank == rootPE_rank) then
      dim_o = o_n%nxyp*max(1, dust_nb)
   else
      dim_o = o_n%mmxp*o_n%mmyp*max(1, dust_nb)
   end if
end if

if (np) nullify(o%dustfi_backup); if (rp) call ptr_release(o%dustfi_backup, e)
if (dust_tog > 0) then
   if (ad_tc) allocate(o%dustfi_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%dustff_backup); if (rp) call ptr_release(o%dustff_backup, e)
if (dust_tog > 0) then
   if (ad_tc) allocate(o%dustff_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%dustaj_backup); if (rp) call ptr_release(o%dustaj_backup, e)
if (dust_tog > 0) then
   if (ad_tc) allocate(o%dustaj_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%dustag_backup); if (rp) call ptr_release(o%dustag_backup, e)
if (dust_tog > 0) then
   if (ad_tc) allocate(o%dustag_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%dustrs_backup); if (rp) call ptr_release(o%dustrs_backup, e)
if (dust_tog > 0) then
   if (ad_tc) allocate(o%dustrs_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (ad_tc) then
   if (my_rank == rootPE_rank) then
      dim_o = o_n%nxyp*max(1, H2Oi_nb)
   else
      dim_o = o_n%mmxp*o_n%mmyp*max(1, H2Oi_nb)
   end if
end if

if (np) nullify(o%H2Oifi_backup); if (rp) call ptr_release(o%H2Oifi_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Oifi_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%H2Oiff_backup); if (rp) call ptr_release(o%H2Oiff_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Oiff_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%H2Oiaj_backup); if (rp) call ptr_release(o%H2Oiaj_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Oiaj_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%H2Oiag_backup); if (rp) call ptr_release(o%H2Oiag_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Oiag_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%H2Oirs_backup); if (rp) call ptr_release(o%H2Oirs_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Oirs_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%H2Odfi_backup); if (rp) call ptr_release(o%H2Odfi_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Odfi_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%H2Odff_backup); if (rp) call ptr_release(o%H2Odff_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Odff_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%H2Odaj_backup); if (rp) call ptr_release(o%H2Odaj_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Odaj_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%H2Odag_backup); if (rp) call ptr_release(o%H2Odag_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Odag_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%H2Odrs_backup); if (rp) call ptr_release(o%H2Odrs_backup, e)
if (H2Oi_tog > 0) then
   if (ad_tc) allocate(o%H2Odrs_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (ad_tc) then
   if (my_rank == rootPE_rank) then
      dim_o = o_n%nxyp*max(1, CO2i_nb)
   else
      dim_o = o_n%mmxp*o_n%mmyp*max(1, CO2i_nb)
   end if
end if

if (np) nullify(o%CO2ifi_backup); if (rp) call ptr_release(o%CO2ifi_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2ifi_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2iff_backup); if (rp) call ptr_release(o%CO2iff_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2iff_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2iaj_backup); if (rp) call ptr_release(o%CO2iaj_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2iaj_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2iag_backup); if (rp) call ptr_release(o%CO2iag_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2iag_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2irs_backup); if (rp) call ptr_release(o%CO2irs_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2irs_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2dfi_backup); if (rp) call ptr_release(o%CO2dfi_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2dfi_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2dff_backup); if (rp) call ptr_release(o%CO2dff_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2dff_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2daj_backup); if (rp) call ptr_release(o%CO2daj_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2daj_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2dag_backup); if (rp) call ptr_release(o%CO2dag_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2dag_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2drs_backup); if (rp) call ptr_release(o%CO2drs_backup, e)
if (CO2i_tog > 0) then
   if (ad_tc) allocate(o%CO2drs_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2wfi_backup); if (rp) call ptr_release(o%CO2wfi_backup, e)
if (CO2i_tog > 0 .and. H2Oi_tog > 0) then
   if (ad_tc) allocate(o%CO2wfi_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2wff_backup); if (rp) call ptr_release(o%CO2wff_backup, e)
if (CO2i_tog > 0 .and. H2Oi_tog > 0) then
   if (ad_tc) allocate(o%CO2wff_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2waj_backup); if (rp) call ptr_release(o%CO2waj_backup, e)
if (CO2i_tog > 0 .and. H2Oi_tog > 0) then
   if (ad_tc) allocate(o%CO2waj_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2wag_backup); if (rp) call ptr_release(o%CO2wag_backup, e)
if (CO2i_tog > 0 .and. H2Oi_tog > 0) then
   if (ad_tc) allocate(o%CO2wag_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (np) nullify(o%CO2wrs_backup); if (rp) call ptr_release(o%CO2wrs_backup, e)
if (CO2i_tog > 0 .and. H2Oi_tog > 0) then
   if (ad_tc) allocate(o%CO2wrs_backup(dim_o), stat=e)
end if
e_check = e_check .or. (e /= 0)

if (e_check) call exception(procedure_name, FATAL,  &
     '{pd-DEPENDENT} State copy 2D aerosol arrays {nxyp,*}')

!== {pd-DEPENDENT} State copy 2D "atmospheric" arrays {nxyp}:

if (ad_tc) then
   if (my_rank == rootPE_rank) then
      dim_o = o_n%nxyp
   else
      dim_o = o_n%mmxp*o_n%mmyp
   end if
end if

if (np) nullify(o%qfz_backup); if (rp) call ptr_release(o%qfz_backup, e)
if (ad_tc) allocate(o%qfz_backup(dim_o), stat=e)
e_check = e_check .or. (e /= 0)

if (e_check) call exception(procedure_name, FATAL,  &
     '{pd-DEPENDENT} State copy 2D atmospheric arrays {nxyp}')

!== {pd-DEPENDENT} State copy 3D sub-/super-surface arrays {nxys3p,patch}:

if (ad_tc) then
   if (my_rank == rootPE_rank) then
      dim_o = o_n%nxys3p
   else
      dim_o = (nzg+nzs)*o_n%mmxp*o_n%mmyp*npatch
   end if
end if

if (np) nullify(o%SBPSH2Oa_backup); if (rp) call ptr_release(o%SBPSH2Oa_backup, e)
if (ad_tc) allocate(o%SBPSH2Oa_backup(dim_o), stat=e)
e_check = e_check .or. (e /= 0)

if (e_check) call exception(procedure_name, FATAL,  &
     '{pd-DEPENDENT} State copy 3D sub-/super-surface arrays {nxys3p,patch}')

!== {pd-DEPENDENT} State copy 2D sub-/super-surface arrays {nxys2p,patch}:

if (ad_tc) then
   if (my_rank == rootPE_rank) then
      dim_o = o_n%nxys2p
   else
      dim_o = o_n%mmxp*o_n%mmyp*npatch
   end if
end if

if (np) nullify(o%sfcH2Omr_backup); if (rp) call ptr_release(o%sfcH2Omr_backup, e)
if (ad_tc) allocate(o%sfcH2Omr_backup(dim_o), stat=e)
e_check = e_check .or. (e /= 0)

if (e_check) call exception(procedure_name, FATAL,  &
     '{pd-DEPENDENT} State copy 2D sub-/super-surface arrays {nxys2p,patch}')

end subroutine manage_grid_object_bspec


end module main_memory_bspec_mod
