!```````````````````````````````````````````````````````````````````
module namelist_bspec_mod

!+ Module that contains routines and variables related to the PRAMS namelist
!  (for Mars)

use univ_kind_defs_mod, only: f4
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN, MAX_MESSAGE_LEN
use univ_integer_time_mod, only: time_obj
use namelist_maxdims_mod, only: nml_max_n_zsubsfc, nml_max_n_sounding,  &
     nml__strtoklen_max
use main_memory_bspec_mod, only: tautot, dust_rta, H2Oi_rta, CO2i_rta,  &
     bdst_tog, bdst_imr, bdst_igd, bdst_icn, bdst_irt, use_gas_opacities,  &
     bdst_iht, bdst_itt, H2Ov_tog, H2Ov_ivr, dust_tog, dust_nb, dust_rmn,  &
     dust_mrt, dust_jef, dust_jth, dust_ivr, dust_igd, dust_imr, dust_inn,  &
     dust_icn, dust_irt, dust_iht, dust_itt, H2Oi_tog, H2Oi_nb, H2Oi_rmn,  &
     H2Oi_mrt, CO2i_tog, CO2i_nb, CO2i_rmn, CO2i_mrt, CO2_sfci, sfci_ils,  &
     nslcon,  nsndp, ipsflg, itsflg, irtsflg, iusflg, H2Ov_beg, ishadtog,  &
     lfmcut, nrdlevt, pctlcon, use_ext_sounding_file, input_sounding_fn,  &
     Ls_of_periapse, lonrad, orbital_eccentricity, axial_obliquity,  &
     AU_to_m_knob, conrnu, RTc_tog, RT_flux_trace_np, dust_op_case,  &
     dust_ctau_refp

implicit none

character(len=4), parameter :: ID_string_bspec = 'Mars'

character(len=MAX_FILE_PATH_LEN) :: default_input_data_dir_bspec

!== &namelist_info_bspec
integer, parameter :: n_chars_nml_version = 4, n_chars_nml_is_for = 5
character(len=n_chars_nml_is_for) :: namelist_is_for
character(len=n_chars_nml_version) :: namelist_version_Mars
character(len=MAX_MESSAGE_LEN) :: namelist_notes_Mars
namelist /namelist_info_bspec/ namelist_is_for, namelist_version_Mars,  &
     namelist_notes_Mars

!== &model_options_bspec
character(len=nml__strtoklen_max), dimension(nml_max_n_sounding) ::  &
     RT_flux_trace_pt_
character(len=nml__strtoklen_max), dimension(2) :: RT_flux_trace_ti_
character(len=nml__strtoklen_max) :: dust_rtt_, H2Oi_rtt_, CO2i_rtt_,  &
     dust_beg_, dust_jsr_, dust_jbg_, H2Oi_beg_, CO2i_beg_, H2Ov_beg_
type(time_obj) :: dust_rtt_Time, H2Oi_rtt_Time, CO2i_rtt_Time, dust_beg_Time,  &
     dust_jsr_Time, dust_jbg_Time, H2Oi_beg_Time, CO2i_beg_Time, H2Ov_beg_Time
type(time_obj), dimension(2) :: RT_flux_trace_ti_Time
real(f4), dimension(nml_max_n_zsubsfc) :: slz_, stgoff_, slmstr_
namelist /model_options_bspec/ pctlcon, CO2_sfci, sfci_ils, nslcon, slz_,  &
     stgoff_, slmstr_, rtc_tog, lonrad, ishadtog, Ls_of_periapse,  &
     orbital_eccentricity, axial_obliquity, AU_to_m_knob, RT_flux_trace_np,  &
     RT_flux_trace_ti_, RT_flux_trace_pt_, lfmcut, nrdlevt, tautot, conrnu,  &
     dust_op_case, use_gas_opacities,  &
     dust_rta,  dust_rtt_, H2Oi_rta, H2Oi_rtt_, CO2i_rta, CO2i_rtt_,  &
     dust_ctau_refp,  &
     bdst_tog, bdst_imr, bdst_igd, bdst_icn, bdst_irt, bdst_iht, bdst_itt,   &
     H2Ov_tog, H2Ov_beg_, H2Ov_ivr, dust_tog, dust_nb, dust_rmn,  &
     dust_mrt, dust_beg_, dust_jsr_, dust_jbg_, dust_jef, dust_jth, dust_ivr,  &
     dust_igd, dust_imr, dust_inn, dust_icn, dust_irt, dust_iht, dust_itt,  &
     H2Oi_tog, H2Oi_nb, H2Oi_rmn, H2Oi_mrt, H2Oi_beg_, CO2i_tog, CO2i_nb,  &
     CO2i_rmn, CO2i_mrt, CO2i_beg_

!== &initial_sounding
real(f4), dimension(nml_max_n_sounding) :: hsnd_, psnd_, tsnd_, rtsnd_,  &
     usnd_, vsnd_ 
namelist /initial_sounding/ nsndp, use_ext_sounding_file, input_sounding_fn,  &
     ipsflg, itsflg, irtsflg, iusflg, hsnd_, psnd_, tsnd_, rtsnd_, usnd_, vsnd_


contains


!```````````````````````````````````````````````````````````````````
subroutine nml_info_check_bspec(namelist_fh)

!+ Check '&namelist_info_bspec' namelist block for errors 

use univ_std_fileIO_mod, only: find_file_IOunit
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN, MAX_PROC_NAME_LEN
use univ_namelist_utils_mod, only: nml_num_value_check

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh    ! File I/O unit of open namelist file

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'nml_info_check_bspec'

character(len=n_chars_nml_is_for) :: base_app_to_match = 'PRAMS'
character(len=n_chars_nml_version) :: base_version_to_match = '0006'

integer :: cn, IO_status
character(len=MAX_MESSAGE_LEN) :: msg

!=== Executable statements:

! Input &namelist_info_bspec namelist block; check for errors:

read(namelist_fh, nml=namelist_info_bspec, iostat=IO_status)
if (IO_status /= 0) call exception(procedure_name, FATAL,  &
     'Namelist block &namelist_info_bspec not found or is damaged (incorrect target body?).')

cn = max(1, len_trim(namelist_is_for))
if (namelist_is_for(1:cn) /= base_app_to_match) then
   write(msg, '(5a)')  'Input namelist application (', namelist_is_for(1:cn),  &
        ') is incompatible with the necessary application (',  &
        base_app_to_match, ').'
   call exception(procedure_name, FATAL, trim(msg))
end if
cn = max(1, len_trim(namelist_version_Mars))
if (namelist_version_Mars(1:cn) /= base_version_to_match) then
   write(msg, '(5a)')  'Input namelist version (',  &
        namelist_version_Mars(1:cn),  &
        ') is incompatible with the necessary version (',  &
        base_version_to_match, ').'
   call exception(procedure_name, FATAL, trim(msg))
end if

end subroutine nml_info_check_bspec


!```````````````````````````````````````````````````````````````````
subroutine proc_nml_basic_control_bspec(namelist_fh)

!+ Processing related to items from namelist &basic_control_bspec

!%use univ_exception_mod, only: exception, FATAL
!%use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use main_memory_bspec_mod, only: manage_nml_var_memory_bspec  ! and others already used by this module

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh

!== Local declarations:

!%character(len=MAX_PROC_NAME_LEN), parameter ::  &
!%     procedure_name = 'proc_nml_basic_control_bspec'

!%integer :: IO_status

!=== Executable statements:

!== Initialize namelist variables with out of range values (where possible) for
!    checking purposes:

! NONE, yet

!== Attempt to read the namelist:

!%read(namelist_fh, nml=basic_control_bspec, iostat=IO_status)
!%if (IO_status /= 0) call exception(procedure_name, FATAL,  &
!%     'Namelist block &basic_control_bspec not found or is damaged.')

!== Check whether some namelist values are within an allowable range:

call manage_nml_var_memory_bspec(dim_n_grids=.TRUE.) ! Allocate grid object(s)

end subroutine proc_nml_basic_control_bspec


!```````````````````````````````````````````````````````````````````
subroutine proc_nml_inout_config_bspec(namelist_fh)

!+ Processing related to items from namelist &inout_config_bspec

!%use univ_exception_mod, only: exception, FATAL
!%use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use main_memory_mod, only:  &  ! and others already used by this module
     default_input_data_dir

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh

!== Local declarations:

!%character(len=MAX_PROC_NAME_LEN), parameter ::  &
!%     procedure_name = 'proc_nml_inout_config_bspec'

!%integer :: IO_status

!=== Executable statements:

!== Initialize namelist variables with out of range values (where possible) for
!    checking purposes:

! NONE, yet

!== Attempt to read the namelist:

!%read(namelist_fh, nml=inout_config_bspec, iostat=IO_status)
!%if (IO_status /= 0) call exception(procedure_name, FATAL,  &
!%     'Namelist block &inout_config_bspec not found or is damaged.')

!== Check whether some namelist values are within an allowable range:

default_input_data_dir_bspec =  &
     trim(default_input_data_dir)//'/'//ID_string_bspec

end subroutine proc_nml_inout_config_bspec


!```````````````````````````````````````````````````````````````````
subroutine proc_nml_model_options_bspec(namelist_fh)

!+ Processing related to items from namelist &model_options_bspec

use univ_math_constants_mod, only: ONE_f4, ZERO_f4
use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_integer_time_mod, only: time_eqv_fp_seconds
use univ_numtype_constants_mod, only: epsilon_f4, tiny_f4
use univ_namelist_utils_mod, only: gen_time_strrep_to_Time, nml_num_value_check
use main_memory_bspec_mod, only:  & ! and others already used by this module
     manage_nml_var_memory_bspec, slz, stgoff, slmstr, dust_rtt,  &
     H2Oi_rtt, CO2i_rtt, dust_jsr, dust_jbg, dust_beg, H2Oi_beg, CO2i_beg,  &
     RT_flux_trace_ti, RT_flux_trace_pt
use namelist_utils_mod, only: strrep_to_trace_coord
use main_memory_mod, only: nzg, mnsdy_Time, morbpd_Time, module_tps

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'proc_nml_model_options_bspec'

integer :: IO_status, i
logical :: error

!=== Executable statements:

call manage_nml_var_memory_bspec(dim_nzg=.TRUE.)  ! Alloc nml vars with dim nzg

!== Initialize namelist variables with out of range values (where possible) for
!    checking purposes:

pctlcon = -ONE_f4
CO2_sfci = -1; sfci_ils = -ONE_f4; nslcon = -1; slz_(:) = ONE_f4
stgoff_(:) = ZERO_f4; slmstr_(:) = ZERO_f4
rtc_tog = -1; lonrad = -1; ishadtog = -1
Ls_of_periapse = -ONE_f4; orbital_eccentricity = -ONE_f4
axial_obliquity = -999._f4; AU_to_m_knob = -ONE_f4
RT_flux_trace_np = -1; RT_flux_trace_ti_(:) = '__NULL__'
RT_flux_trace_pt_(:) = '__NULL__'
lfmcut = -1; nrdlevt = -1; tautot = -ONE_f4; conrnu = ZERO_f4
dust_op_case = -1; use_gas_opacities = .TRUE.
dust_rta = .FALSE.; dust_rtt_ = '__NULL__'
H2Oi_rta = .FALSE.; H2Oi_rtt_ = '__NULL__'
CO2i_rta = .FALSE.; CO2i_rtt_ = '__NULL__'
dust_ctau_refp = -999._f4
bdst_tog = -1; bdst_imr = -ONE_f4; bdst_igd = -ONE_f4; bdst_icn = -ONE_f4
bdst_irt = -ONE_f4; bdst_iht = -ONE_f4; bdst_itt = -ONE_f4
H2Ov_tog = -1; H2Ov_beg_ = '__NULL__'; H2Ov_ivr = -1
dust_tog = -1; dust_nb = -1; dust_rmn = -ONE_f4; dust_mrt = -ONE_f4
dust_beg_ = '__NULL__'; dust_jsr_ = '__NULL__'; dust_jbg_ = '__NULL__'
dust_jef = -ONE_f4; dust_jth = -ONE_f4; dust_ivr = -1; dust_igd = -ONE_f4
dust_imr = -ONE_f4; dust_inn = -ONE_f4; dust_icn = -ONE_f4
dust_irt = -ONE_f4; dust_iht = -ONE_f4; dust_itt = -ONE_f4
H2Oi_tog = -1; H2Oi_nb = -1; H2Oi_rmn = -ONE_f4; H2Oi_mrt = -ONE_f4
H2Oi_beg_ = '__NULL__'
CO2i_tog = -1; CO2i_nb = -1; CO2i_rmn = -ONE_f4; CO2i_mrt = -ONE_f4
CO2i_beg_ = '__NULL__'

!== Attempt to read the namelist:

read(namelist_fh, nml=model_options_bspec, iostat=IO_status)
if (IO_status /= 0) call exception(procedure_name, FATAL,  &
     'Namelist block &model_options_bspec not found or is damaged.')

!== Check whether some namelist values are within an allowable range:

call nml_num_value_check(pctlcon, 'PCTLCON', 'and', gt=ONE_f4-epsilon_f4,  &
     lt=ONE_f4+epsilon_f4)

call nml_num_value_check(CO2_sfci, 'CO2_SFCI', 'and', ge=1, le=3)
call nml_num_value_check(sfci_iLs, 'SFCI_ILS', 'and', gt=(-tiny_f4),  &
     lt=360._f4+epsilon_f4)

call nml_num_value_check(nslcon, 'NSLCON', ge=0, le=1)
call nml_num_value_check(slz_(1:nzg), 'SLZ_', le=(-tiny_f4))
slz(1:nzg) = slz_(1:nzg)
call nml_num_value_check(stgoff_(1:nzg), 'STGOFF_')
stgoff(1:nzg) = stgoff_(1:nzg)
call nml_num_value_check(slmstr_(1:nzg), 'SLMSTR_')
slmstr(1:nzg) = slmstr_(1:nzg)

call nml_num_value_check(RTc_tog, 'RTC_TOG', 'and', ge=1, le=4)
call nml_num_value_check(lonrad, 'LONRAD', 'and', ge=0, le=1)
call nml_num_value_check(ishadtog, 'ISHADTOG', 'and', ge=0, le=1)

call nml_num_value_check(Ls_of_periapse, 'LS_OF_PERIAPSE', 'and', ge=ZERO_f4,  &
     lt=360._f4)
call nml_num_value_check(orbital_eccentricity, 'ORBITAL_ECCENTRICITY', 'and',  &
     ge=ZERO_f4, lt=ONE_f4)
call nml_num_value_check(axial_obliquity, 'AXIAL_OBLIQUITY', 'and',  &
     ge=-180._f4, le=180._f4)
call nml_num_value_check(AU_to_m_knob, 'AU_TO_M_KNOB', 'and', gt=0.999_f4,  &
     lt=1.001_f4)

call nml_num_value_check(RT_flux_trace_np, 'RT_FLUX_TRACE_NP', 'and', ge=0,  &
     le=nml_max_n_sounding)
if (RT_flux_trace_np > 0) then    ! Only perform if needed
   call manage_nml_var_memory_bspec(dim_RT_flux_trace_np=.TRUE.)
   do i=1,2
      call gen_time_strrep_to_Time(trim(RT_flux_trace_ti_(i)),  &
           RT_flux_trace_ti_Time(i), error, mnsdy_Time, morbpd_Time, module_tps)
      if (error) call exception(procedure_name, FATAL,  &
           'Improper value for namelist variable RT_FLUX_TRACE_TI_')
      RT_flux_trace_ti(i) = time_eqv_fp_seconds(RT_flux_trace_ti_Time(i))
   end do
   do i=1,RT_flux_trace_np
      call strrep_to_trace_coord(trim(RT_flux_trace_pt_(i)),  &
           RT_flux_trace_pt(i), error)
      if (error) call exception(procedure_name, FATAL,  &
           'Improper value for namelist variable RT_FLUX_TRACE_PT_')
   end do
end if

call nml_num_value_check(lfmcut, 'LFMCUT', ge=1)
call nml_num_value_check(nrdlevt, 'NRDLEVT', ge=1)

call nml_num_value_check(dust_op_case, 'DUST_OP_CASE', 'and', ge=1, le=3)

call gen_time_strrep_to_Time(trim(dust_rtt_), dust_rtt_Time, error,  &
     mnsdy_Time, morbpd_Time, module_tps)
if (error) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable DUST_RTT_')
dust_rtt = time_eqv_fp_seconds(dust_rtt_Time)

call gen_time_strrep_to_Time(trim(H2Oi_rtt_), H2Oi_rtt_Time, error,  &
     mnsdy_Time, morbpd_Time, module_tps)
if (error) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable H2OI_RTT_')
H2Oi_rtt = time_eqv_fp_seconds(H2Oi_rtt_Time)

call gen_time_strrep_to_Time(trim(CO2i_rtt_), CO2i_rtt_Time, error,  &
     mnsdy_Time, morbpd_Time, module_tps)
if (error) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable CO2I_RTT_')
CO2i_rtt = time_eqv_fp_seconds(CO2i_rtt_Time)

call nml_num_value_check(dust_ctau_refp, 'DUST_CTAU_REFP', gt=tiny_f4)

call nml_num_value_check(bdst_tog, 'BDST_TOG', 'and', ge=0, le=4)
call nml_num_value_check(bdst_imr, 'BDST_IMR', ge=tiny_f4)
call nml_num_value_check(bdst_igd, 'BDST_IGD', gt=ONE_f4-epsilon_f4)
call nml_num_value_check(bdst_icn, 'BDST_ICN', ge=tiny_f4)
call nml_num_value_check(bdst_irt, 'BDST_IRT', gt=(-tiny_f4))
call nml_num_value_check(bdst_iht, 'BDST_IHT', ge=tiny_f4)
call nml_num_value_check(bdst_itt, 'BDST_ITT', gt=(-tiny_f4))

call nml_num_value_check(H2Ov_tog, 'H2OV_TOG', 'and', ge=0, le=3)
call gen_time_strrep_to_Time(trim(H2Ov_beg_), H2Ov_beg_Time, error,  &
     mnsdy_Time, morbpd_Time, module_tps)
if (error) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable H2OV_BEG_')
H2Ov_beg = time_eqv_fp_seconds(H2Ov_beg_Time)
call nml_num_value_check(H2Ov_ivr, 'H2OV_IVR', 'and', ge=0, le=1)

call nml_num_value_check(dust_tog, 'DUST_TOG', 'and', ge=0, le=4)
call nml_num_value_check(dust_nb, 'DUST_NB', ge=1)
call nml_num_value_check(dust_rmn, 'DUST_RMN', ge=tiny_f4)
call nml_num_value_check(dust_mrt, 'DUST_MRT', ge=tiny_f4)
call gen_time_strrep_to_Time(trim(dust_beg_), dust_beg_Time, error,  &
     mnsdy_Time, morbpd_Time, module_tps)
if (error) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable DUST_BEG_')
dust_beg = time_eqv_fp_seconds(dust_beg_Time)
call gen_time_strrep_to_Time(trim(dust_jsr_), dust_jsr_Time, error,  &
     mnsdy_Time, morbpd_Time, module_tps)
if (error) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable DUST_JSR_')
dust_jsr = time_eqv_fp_seconds(dust_jsr_Time)
call gen_time_strrep_to_Time(trim(dust_jbg_), dust_jbg_Time, error,  &
     mnsdy_Time, morbpd_Time, module_tps)
if (error) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable DUST_JBG_')
dust_jbg = time_eqv_fp_seconds(dust_jbg_Time)
call nml_num_value_check(dust_jef, 'DUST_JEF', ge=tiny_f4)
call nml_num_value_check(dust_jth, 'DUST_JTH', ge=tiny_f4)
call nml_num_value_check(dust_ivr, 'DUST_IVR', 'and', ge=0, le=5)
call nml_num_value_check(dust_igd, 'DUST_IGD', gt=ONE_f4-epsilon_f4)
call nml_num_value_check(dust_imr, 'DUST_IMR', ge=tiny_f4)
call nml_num_value_check(dust_inn, 'DUST_INN', gt=(-tiny_f4))
call nml_num_value_check(dust_icn, 'DUST_ICN', ge=tiny_f4)
call nml_num_value_check(dust_irt, 'DUST_IRT', gt=(-tiny_f4))
call nml_num_value_check(dust_iht, 'DUST_IHT', ge=tiny_f4)
call nml_num_value_check(dust_itt, 'DUST_ITT', gt=(-tiny_f4))

call nml_num_value_check(H2Oi_tog, 'H2OI_TOG', 'and', ge=0, le=2)
call nml_num_value_check(H2Oi_nb, 'H2OI_NB', ge=1)
call nml_num_value_check(H2Oi_rmn, 'H2OI_RMN', ge=tiny_f4)
call nml_num_value_check(H2Oi_mrt, 'H2OI_MRT', ge=tiny_f4)
call gen_time_strrep_to_Time(trim(H2Oi_beg_), H2Oi_beg_Time, error,  &
     mnsdy_Time, morbpd_Time, module_tps)
if (error) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable H2OI_BEG_')
H2Oi_beg = time_eqv_fp_seconds(H2Oi_beg_Time)

call nml_num_value_check(CO2i_tog, 'CO2I_TOG', 'and', ge=0, le=2)
call nml_num_value_check(CO2i_nb, 'CO2I_NB', ge=1)
call nml_num_value_check(CO2i_rmn, 'CO2I_RMN', ge=tiny_f4)
call nml_num_value_check(CO2i_mrt, 'CO2I_MRT', ge=tiny_f4)
call gen_time_strrep_to_Time(trim(CO2i_beg_), CO2i_beg_Time, error,  &
     mnsdy_Time, morbpd_Time, module_tps)
if (error) call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable CO2I_BEG_')
CO2i_beg = time_eqv_fp_seconds(CO2i_beg_Time)

end subroutine proc_nml_model_options_bspec


!```````````````````````````````````````````````````````````````````
subroutine proc_nml_initial_sndg(namelist_fh)

!+ Processing related to items from namelist &initial_sounding

use univ_exception_mod, only: exception, FATAL
use univ_math_constants_mod, only: ONE_f4
use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use univ_numtype_constants_mod, only: epsilon_f4, tiny_f4
use univ_namelist_utils_mod, only: nml_num_value_check
use main_memory_bspec_mod, only:  &
     manage_nml_var_memory_bspec, hsnd,  &  ! and others already used by this module
     psnd, tsnd, rtsnd, usnd, vsnd

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'proc_nml_initial_sndg'

integer :: IO_status

!=== Executable statements:

!== Initialize namelist variables with out of range values (where possible) for
!    checking purposes:

nsndp = -1; use_ext_sounding_file = .FALSE.; input_sounding_fn = '__NULL__'
ipsflg = -1; itsflg = -1; irtsflg = -1; iusflg = -1;
hsnd_(:) = -ONE_f4; psnd_(:) = -ONE_f4; tsnd_(:) = -ONE_f4
rtsnd_(:) = -ONE_f4; usnd_(:) = -ONE_f4; vsnd_(:) = -ONE_f4

!== Attempt to read the namelist:

read(namelist_fh, nml=initial_sounding, iostat=IO_status)
if (IO_status /= 0) call exception(procedure_name, FATAL,  &
     'Namelist block &initial_sounding not found or is damaged.')

!== Check whether some namelist values are within an allowable range:

call nml_num_value_check(nsndp, 'NSNDP', 'and', ge=1, le=nml_max_n_sounding)
call manage_nml_var_memory_bspec(dim_nsndp = .TRUE.)

!... USE_EXT_SOUNDING_FILE (logical) only has two possible values
if (use_ext_sounding_file .and. trim(input_sounding_fn) == '__NULL__')  &
     call exception(procedure_name, FATAL,  &
     'Improper value for namelist variable INPUT_SOUNDING_FN')
call nml_num_value_check(ipsflg, 'IPSFLG', 'and', ge=0, le=2)
call nml_num_value_check(itsflg, 'ITSFLG', 'and', ge=0, le=1)
call nml_num_value_check(irtsflg, 'IRTSFLG', 'and', ge=0, le=1)
call nml_num_value_check(iusflg, 'IUSFLG', 'and', ge=0, le=1)

if (.not.use_ext_sounding_file) then   ! Check "inline" sounding values
   select case (ipsflg)
   case(0)
      call nml_num_value_check(hsnd_(1:nsndp), 'HSND_', ge=(-tiny_f4))
      call nml_num_value_check(psnd_(1:nsndp), 'PSND_', ge=tiny_f4)
      psnd(1:nsndp) = psnd_(1:nsndp)
   case(1)
      call nml_num_value_check(hsnd_(1:nsndp), 'HSND_', ge=(-tiny_f4))
      call nml_num_value_check(psnd_(1), 'PSND_', ge=tiny_f4)
      psnd(1) = psnd_(1)
   case(2)
      call nml_num_value_check(psnd_(1:nsndp), 'PSND_', ge=tiny_f4)
      psnd(1:nsndp) = psnd_(1:nsndp)
   end select
   hsnd(1:nsndp) = hsnd_(1:nsndp)

   call nml_num_value_check(tsnd_(1:nsndp), 'TSND_', ge=tiny_f4)
   tsnd(1:nsndp) = tsnd_(1:nsndp)
   call nml_num_value_check(rtsnd_(1:nsndp), 'RTSND_', ge=(-tiny_f4))
   rtsnd(1:nsndp) = rtsnd_(1:nsndp)

   select case (iusflg)
   case(0)
      call nml_num_value_check(usnd_(1:nsndp), 'USND_')
      call nml_num_value_check(usnd_(1:nsndp), 'VSND_')
   case(1)
      call nml_num_value_check(usnd_(1:nsndp), 'USND_', 'and', gt=(-tiny_f4),  &
           lt=360._f4+epsilon_f4)
      call nml_num_value_check(vsnd_(1:nsndp), 'VSND_', gt=-(tiny_f4))
   end select
   usnd(1:nsndp) = usnd_(1:nsndp)
   vsnd(1:nsndp) = vsnd_(1:nsndp)
end if

end subroutine proc_nml_initial_sndg


end module namelist_bspec_mod
