"""
This is a python-syntax configuration file for template substitutions.

The python-syntax blocks below define substitution keywords and their values
(over multiple iterations) -- for use with the associated template (*.template)
and 'process_template_file.py' (which uses the standard string.Template class),
producing the final associated file(s).
"""

# Output file text encoding -- set to
#  any valid Python codec name -OR- "" (uses default encoding for locale)
#   NOTE: for Fortran source code, set to "ascii"
output_f_codec = "ascii"

# A dictionary of keys with only scalar values that will be used to substitute
# into the given values of 'sdl_string' and 'output_fn_template'.

defines_dict = dict( \
                   common_path = "%SUBFILE%:namelist_path-common",
                   bspec_path = "%SUBFILE%:namelist_path-bspec" \
                   )

# A quoted list of dictionaries which have keys with lists as their values.  It
# must remain quoted (rendering it as a long string) to permit substitution of
# any tokens within it using the 'defines_dict' dictionary.  Each of the value
# elements of each dictionary key implies an iteration.  Multiple (up to 3)
# dictionaries can be used to produce even more complex output.  A single
# dictionary with key values having 3 elements will generate 3 iterations of
# template substitution.  Two dictionaries with 3 and 4 elements, respectively,
# will generate 12 iterations of template substitution.
#
# Every key list below (i.e., within the '[]' after each '=') must have the same
# number of elements.  Only strings allowed (e.g., no unquoted numeric values).

sdl_string = '''[ \
    dict( \
        namelist_info = ["%SUBFILE%:${common_path}/PRAMS_IN-namelist_info.in"],
        namelist_info_bspec = ["%SUBFILE%:${bspec_path}/PRAMS_IN-namelist_info_bspec.in"],
        basic_control = ["%SUBFILE%:${common_path}/PRAMS_IN-basic_control.in"],
        time_config = ["%SUBFILE%:${common_path}/PRAMS_IN-time_config.in"],
        spatial_config = ["%SUBFILE%:${common_path}/PRAMS_IN-spatial_config.in"],
        inout_config = ["%SUBFILE%:${common_path}/PRAMS_IN-inout_config.in"],
        model_options = ["%SUBFILE%:${common_path}/PRAMS_IN-model_options.in"],
        model_options_bspec = ["%SUBFILE%:${bspec_path}/PRAMS_IN-model_options_bspec.in"],
        initial_sounding = ["%SUBFILE%:${bspec_path}/PRAMS_IN-initial_sounding.in"],
        PIPP_control = ["%SUBFILE%:${common_path}/PRAMS_IN-PIPP_control.in"] \
        ) ]'''

# A quoted string denoting the output filename (with path, if desired).  This
# will undergo substitution of any tokens within it using both the
# 'defines_dict' dictionary and a single iteration of the 'sdl_string'
# dictionary.

output_fn_template = \
        '''PRAMS_IN'''
