!```````````````````````````````````````````````````````````````````
module postp_namelist_bspec_mod

!+ Module that contains body-specific routines related to the postp namelist

use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN, MAX_FILE_PATH_LEN
use namelist_bspec_mod, only: ID_string_bspec

implicit none

character(len=MAX_FILE_PATH_LEN) :: default_input_data_dir_bspec

!== &namelist_info_bspec
integer, parameter :: n_chars_nml_version = 4, n_chars_nml_is_for = 19
character(len=n_chars_nml_is_for), public :: namelist_is_for
character(len=n_chars_nml_version), public :: namelist_version_Mars
character(len=MAX_MESSAGE_LEN), public :: namelist_notes_Mars
namelist /namelist_info_bspec/ namelist_is_for, namelist_version_Mars,  &
     namelist_notes_Mars


contains


!```````````````````````````````````````````````````````````````````
subroutine nml_info_check_bspec(namelist_fh)

!+ Check '&namelist_info_bspec' namelist block for errors 

use univ_exception_mod, only: exception, FATAL
use univ_maxlen_string_mod, only: MAX_MESSAGE_LEN, MAX_PROC_NAME_LEN

implicit none

!=== Argument declarations:

integer, intent(IN) :: namelist_fh    ! File I/O unit of open namelist file

!== Local declarations:

character(len=MAX_PROC_NAME_LEN), parameter ::  &
     procedure_name = 'nml_info_check_bspec'

character(len=n_chars_nml_is_for) :: base_app_to_match = 'postp_for_PRAMS'
character(len=n_chars_nml_version) :: base_version_to_match = '0001'

integer :: cn, IO_status
character(len=MAX_MESSAGE_LEN) :: msg

!=== Executable statements:

! Input &namelist_info_bspec namelist block; check for errors:

read(namelist_fh, nml=namelist_info_bspec, iostat=IO_status)
if (IO_status /= 0) call exception(procedure_name, FATAL,  &
     'Namelist block &namelist_info_bspec not found or is damaged (incorrect target body?).')

cn = max(1, len_trim(namelist_is_for))
if (namelist_is_for(1:cn) /= base_app_to_match) then
   write(msg, '(5a)')  'Input namelist application (', namelist_is_for(1:cn),  &
        ') is incompatible with the necessary application (',  &
        base_app_to_match, ').'
   call exception(procedure_name, FATAL, trim(msg))
end if
cn = max(1, len_trim(namelist_version_Mars))
if (namelist_version_Mars(1:cn) /= base_version_to_match) then
   write(msg, '(5a)')  'Input namelist version (',  &
        namelist_version_Mars(1:cn),  &
        ') is incompatible with the necessary version (',  &
        base_version_to_match, ').'
   call exception(procedure_name, FATAL, trim(msg))
end if

end subroutine nml_info_check_bspec


!```````````````````````````````````````````````````````````````````
subroutine proc_nml_pp_basic_ctl_bspec(input_fh)

!+ Processing related to items from namelist &pp_basic_control_bspec

!%use univ_exception_mod, only: exception, FATAL
!%use univ_maxlen_string_mod, only: MAX_PROC_NAME_LEN
use postp_main_memory_mod, only:  &  ! and others already used by this module
     default_input_data_dir
use namelist_bspec_mod, only: didd_bspec=>default_input_data_dir_bspec

implicit none

!=== Argument declarations:

integer, intent(IN) :: input_fh

!== Local declarations:

!%character(len=MAX_PROC_NAME_LEN), parameter ::  &
!%     procedure_name = 'proc_nml_pp_basic_ctl_bspec'

!%integer :: IO_status

!=== Executable statements:

!== Initialize namelist variables with out of range values (where possible) for
!    checking purposes:

! NONE, yet

!== Attempt to read the namelist:

!%read(input_fh, nml=pp_basic_control_bspec, iostat=IO_status)
!%if (IO_status /= 0) call exception(procedure_name, FATAL,  &
!%     'Namelist block &pp_basic_control_bspec not found or is damaged.')

!== Check whether some namelist values are within an allowable range:

default_input_data_dir_bspec =  &
     trim(default_input_data_dir)//'/'//ID_string_bspec
didd_bspec = default_input_data_dir_bspec  ! Fill PRAMS parameter

end subroutine proc_nml_pp_basic_ctl_bspec


end module postp_namelist_bspec_mod
