!###################################################################
!          ** PRAMS Ingestion and Preprocessing Package **
!###################################################################

!```````````````````````````````````````````````````````````````````
program split_UK_ncoutp

!+ Puts UK (Open Univ -- formerly Oxford) MGCM (spectral; NetCDF output format)
!   run data into PIPP format

use univ_kind_defs_mod, only: f4
use univ_std_fileIO_mod, only: find_file_IOunit, determine_IO_reclen
use univ_maxlen_string_mod, only: MAX_FILE_PATH_LEN
use PIPP_format_mod, only: lsubs_ts, utc_ts, lat, lon, topo, rec_size, vtype,  &
     dstring, PIPPf_Nmalloc, PIPPf_w_fill, PIPPf_w_data, PIPPf_w_head,  &
     PIPPf_dealloc, sol_ts, PIPP_data
use main_memory_mod, only: undef_value_f4

implicit none

!== Local declarations:

! Declarations intended to be available "program-wide":

integer :: lat_dimid, lon_dimid, levm_dimid, sbslevm_dimid, rec_dimid

integer :: nsolpf, ntpsl, ntimes
real(f4) :: sptrop, rgas, g, cp, snap_tsep_bmin
integer :: ni, nj, nj1, nj_1, nk, nk_SBS, iv, i, j, k, n, it, IO_reclen

real(f4), allocatable, dimension(:) :: sol_Ls    ! [nsolpf+1]
real(f4), allocatable, dimension(:) :: sigma_levm   ! [nk]
real(f4), allocatable, dimension(:) :: lat0   ! [nj]
real(f4), allocatable, dimension(:) :: lon0   ! [ni]
integer, allocatable, dimension(:) :: nzi,vt    ! [nvar]

real(f4), allocatable, dimension(:,:,:,:,:) :: t_m,  & ! [ni,nj,nk,ntpsl,nsolpf]
     u_m, v_m, H2Ovmr_m, dustmr_m, H2Oimr_m, H2Odmr_m
real(f4), allocatable, dimension(:,:,:,:,:) ::  &  ! [ni,nj,nk_SBS,ntpsl,nsolpf]
     gt_SBS_m
real(f4), allocatable, dimension(:,:,:,:) :: sp_m  &   ! [ni,nj,ntpsl,nsolpf]
     ,gt_sfc_m, gi_sfc_m, ghi_sfc_m, dust_asfc_m, H2Oi_asfc_m, H2Od_asfc_m

real(f4), allocatable, dimension(:,:,:) :: u_a,v_a,p_a,t_a,gpz_a,  & ![ni,nj,nk]
     H2Ovmc_a, dustmc_a, H2Oimc_a, H2Odmc_a, rho_a
real(f4), allocatable, dimension(:,:,:) :: gt_SBS_a       ! [ni,nj,nk_SBS]
real(f4), allocatable, dimension(:,:) :: gt_sfc_a,sp_a,ghi_sfc_a,  &  ! [ni,nj]
     dust_asfc_a, H2Oi_asfc_a, H2Od_asfc_a     
real(f4), allocatable, dimension(:) :: zlevSBS      ! {nk_SBS+1}

integer :: lastslash, fnwp_len, nvar, itime
character(len=128) :: dfname, gradsctlo, PIPPhdr, PIPPdat

integer :: IOunit
logical :: file_exists

character(len=17), parameter :: exe_config_fn = 'auto_gen-prep.nml'

character(len=MAX_FILE_PATH_LEN) :: dfname_wp, outprfx, desc_str
namelist /prep_args/ dfname_wp, outprfx, desc_str

!=== Executable statements:

! Read initial "command-line argument" namelist:

IOunit = find_file_IOunit()   ! Find available IO unit

inquire(file=exe_config_fn, exist=file_exists)
if (.NOT.file_exists) call usage

open (IOunit, file=exe_config_fn, status='old', form='formatted')

read (IOunit, nml=prep_args); close (IOunit)

lastslash = index(dfname_wp,'/',.TRUE.)  ! Find last forward slash in filename
fnwp_len = len_trim(dfname_wp)
dfname = dfname_wp(lastslash+1:fnwp_len)  ! no path

! Create filenames to use later:

write (gradsctlo,'(a,a4)') outprfx(1:len_trim(outprfx)),'.ctl'
write (PIPPhdr,'(a,a4)') outprfx(1:len_trim(outprfx)),'.hdr'
write (PIPPdat,'(a,a4)') outprfx(1:len_trim(outprfx)),'.dat'

! Open header file; read contents:

call rdMGCM_UK_ncoutp(phase=0)

ntimes = nsolpf*ntpsl  ! Number of snapshots per data file

! Set up PIPP variable descriptor arrays:

nvar = 15
allocate(nzi(nvar),vt(nvar))
nzi(1) = nk   ! Temperature [K]
vt(1) = 1     !
nzi(2) = nk  ! Pressure [Pa]
vt(2) = 1    !
nzi(3) = nk   ! U-wind [m/s]
vt(3) = 1     !  
nzi(4) = nk  ! V-wind [m/s]
vt(4) = 1    !
nzi(5) = nk   ! Areopotential height [m]
vt(5) = 1     !
nzi(6) = nk  ! H2O vapor mass concentration [kg/m^3]
vt(6) = 1    !
nzi(7) = nk   ! Dust aerosol mass concentration [kg/m^3]
vt(7) = 1     !
nzi(8) = nk  ! H2O_ice aerosol mass concentration [kg/m^3]
vt(8) = 1    !
nzi(9) = nk   ! H2O_ice (dust core) aerosol mass concentration [kg/m^3]
vt(9) = 1     !
nzi(10) = nk_SBS  ! Subsurface temperature [K]
vt(10) = 3        !
nzi(11) = 1   ! Surface temperature [K] OR -(CO2 sfc ice mass) [kg/m^2]
vt(11) = 4    !
nzi(12) = 1  ! H2O surface ice mass [kg/m^2]
vt(12) = 4   !
nzi(13) = 1   ! dust aerosol surface mass [kg/m^2]
vt(13) = 4    !
nzi(14) = 1  ! H2O ice aerosol surface mass [kg/m^2]
vt(14) = 4   !
nzi(15) = 1   ! H2O ice (dust core) aerosol surface mass [kg/m^2]
vt(15) = 4    !

call PIPPf_Nmalloc(ntimes,ni,nj,nvar,nzi) ! Allocate PIPP_format arrays

dstring(1) = desc_str(1:len(dstring(1)))    ! Set PIPP description string

! Interpolate Ls for all times in file:

write (*,'(/a)') 'Calculating Ls values for entire dataset ...'
call intp_lsubs_new(nsolpf,ntpsl,ntimes,sol_Ls,lsubs_ts(:,1))

! Allocate memory for MGCM fields:

allocate(t_m(ni,nj,nk,ntpsl,nsolpf)  &   ! Arrays for "raw" MGCM values
     ,u_m(ni,nj,nk,ntpsl,nsolpf),v_m(ni,nj,nk,ntpsl,nsolpf)  &
     ,H2Ovmr_m(ni,nj,nk,ntpsl,nsolpf),dustmr_m(ni,nj,nk,ntpsl,nsolpf)  &
     ,gt_SBS_m(ni,nj,nk_SBS,ntpsl,nsolpf),sp_m(ni,nj,ntpsl,nsolpf)  &
     ,ghi_sfc_m(ni,nj,ntpsl,nsolpf),dust_asfc_m(ni,nj,ntpsl,nsolpf)  &
     ,gt_sfc_m(ni,nj,ntpsl,nsolpf),gi_sfc_m(ni,nj,ntpsl,nsolpf))
!     ,H2Oi_asfc_m(ni,nj,ntpsl,nsolpf),H2Od_asfc_m(ni,nj,ntpsl,nsolpf)  &
!     ,H2Oimr_m(ni,nj,nk,ntpsl,nsolpf),H2Odmr_m(ni,nj,nk,ntpsl,nsolpf)  &

allocate(u_a(ni,nj,nk),v_a(ni,nj,nk)  &     ! Arrays for "adjusted" values
     ,p_a(ni,nj,nk),t_a(ni,nj,nk),gpz_a(ni,nj,nk),gt_sfc_a(ni,nj)  &
     ,gt_SBS_a(ni,nj,nk_SBS),sp_a(ni,nj),rho_a(ni,nj,nk),  &
     H2Ovmc_a(ni,nj,nk),dustmc_a(ni,nj,nk),H2Oimc_a(ni,nj,nk),  &
     H2Odmc_a(ni,nj,nk),ghi_sfc_a(ni,nj),dust_asfc_a(ni,nj)  &
     ,H2Oi_asfc_a(ni,nj),H2Od_asfc_a(ni,nj))

! Read in *ALL* relevant MGCM data from data file:

write (*,'(/a/)') 'Reading entire MGCM dataset ...'
call rdMGCM_UK_ncoutp(phase=1)

! Process each timestep in sequence; write PIPP format datafile: 

IO_reclen = determine_IO_reclen(PIPP_data(1:rec_size(1)))

   ! Open PIPP format datafile: 
open(IOunit, file=PIPPdat(1:len_trim(PIPPdat)), status='unknown',  &
     form='unformatted', access='direct', recl=IO_reclen)

do n=1,nsolpf

   do it=1,ntpsl  ! Loop over number of times per sol that MGCM output is made

      itime = (n-1)*ntpsl+it  ! Number of times from data file beginning

      write (*,'(a,i2,a,i2,a,f7.2,a)')  &          ! Report progress to user:
           'Processing and writing PIPP format: SOL=',n,', TIMESTEP=',it  &
           ,', LS=',lsubs_ts(itime,1),' ...'

      sol_ts(itime,1) = n

      u_a(:,:,:) = u_m(:,:,:,it,n); v_a(:,:,:) = v_m(:,:,:,it,n)

      t_a(:,:,:) = t_m(:,:,:,it,n); gt_SBS_a(:,:,:) = gt_SBS_m(:,:,:,it,n)

      do j=1,nj
         do i=1,ni
            if(gi_sfc_m(i,j,it,n) > 1.e-9) then
               gt_sfc_a(i,j) = -gi_sfc_m(i,j,it,n)
            else
               gt_sfc_a(i,j) = gt_sfc_m(i,j,it,n)
            endif
         enddo
      enddo

      ghi_sfc_a(:,:) = ghi_sfc_m(:,:,it,n)
      dust_asfc_a(:,:) = dust_asfc_m(:,:,it,n)
      H2Oi_asfc_a(:,:) = 0._f4  ! H2Oi_asfc_m(:,:,it,n)
      H2Od_asfc_a(:,:) = 0._f4  ! H2Od_asfc_m(:,:,it,n)

      ! Calculate pressures on sigma levels and correct areopotential height:
      sp_a(:,:) = sp_m(:,:,it,n)
      call p_gpz_calc

      rho_a(:,:,:) = p_a(:,:,:)/(rgas*t_a(:,:,:))

      ! Convert mass mixing ratios to mass concentrations:
      H2Ovmc_a(:,:,:) = 0._f4 ! H2Ovmr_m(:,:,:,it,n)*rho_a(:,:,:)
      dustmc_a(:,:,:) = dustmr_m(:,:,:,it,n)*rho_a(:,:,:)
      H2Oimc_a(:,:,:) = 0._f4 ! H2Oimr_m(:,:,:,it,n)*rho_a(:,:,:)
      H2Odmc_a(:,:,:) = 0._f4 ! H2Odmr_m(:,:,:,it,n)*rho_a(:,:,:)

      ! Fill PIPP 4D data cube with one timestep's data:
  
      call PIPPf_w_fill(t_a(:,:,:),1)
      call PIPPf_w_fill(p_a(:,:,:),2)
      call PIPPf_w_fill(u_a(:,:,:),3)
      call PIPPf_w_fill(v_a(:,:,:),4)
      call PIPPf_w_fill(gpz_a(:,:,:),5)
      call PIPPf_w_fill(H2Ovmc_a(:,:,:),6)
      call PIPPf_w_fill(dustmc_a(:,:,:),7)
      call PIPPf_w_fill(H2Oimc_a(:,:,:),8)
      call PIPPf_w_fill(H2Odmc_a(:,:,:),9)
      call PIPPf_w_fill(gt_SBS_a(:,:,:),10)
      call PIPPf_w_fill(gt_sfc_a(:,:),11)
      call PIPPf_w_fill(ghi_sfc_a(:,:),12)
      call PIPPf_w_fill(dust_asfc_a(:,:),13)
      call PIPPf_w_fill(H2Oi_asfc_a(:,:),14)
      call PIPPf_w_fill(H2Od_asfc_a(:,:),15)

      ! Write the timestep's data to PIPP format file:

      call PIPPf_w_data(IOunit, itime)

   end do
end do

close(IOunit)     ! Close PIPP format datafile

! Write PIPP format header file:

do j=1,nj
   do i=1,ni
      lat(i,j,1) = lat0(j)
      lon(i,j,1) = lon0(i)
   enddo
enddo

do iv=1,nvar
   vtype(iv,1) = vt(iv)
enddo

call PIPPf_w_head(PIPPhdr)

! Open special header file; write SBS depth info:

open(IOunit, file=PIPPhdr(1:len_trim(PIPPhdr))//'.tzl', status='unknown',  &
     form='unformatted')
write (IOunit) nk_SBS
write (IOunit) zlevSBS
close(IOunit)

! Write GrADS descriptor file:

open(IOunit, file=gradsctlo(1:len_trim(gradsctlo)), status='unknown',  &
     form='formatted')

lastslash = index(PIPPdat,'/',.TRUE.)  ! Find last forward slash in filename
fnwp_len = len_trim(PIPPdat)
dfname = PIPPdat(lastslash+1:fnwp_len)  ! no path
write (IOunit, '(a,2x,a)') 'dset', '^'//dfname(1:len_trim(dfname))

write (IOunit, '(a,2x,e13.6)') 'undef', undef_value_f4
write (IOunit, '(a)') 'title MGCM data'
write (IOunit, '(a,2x,i3,2x,a,2x,f6.1,2x,f6.1)') 'xdef', ni, 'linear',  &
     lon0(1), 360._f4/real(ni, kind=f4)
write (IOunit, '(a,2x,i3,2x,a,2x,f6.1,2x,f6.1)') 'ydef', nj, 'linear',  &
     lat0(1), 180._f4/real(nj, kind=f4)
write (IOunit, '(a,2x,i3,2x,a,2x,i1,2x,i1)') 'zdef', nk, 'linear', 1, 1
i = max(1, nint(snap_tsep_bmin))  ! [body-minutes]
if (i <= 9) then
   write (IOunit, '(a,2x,i4,2x,a,i1,a)') 'tdef', ntimes,  &
        'linear 00:00z04jul1997  ', i, 'mn'
else if (i <= 99) then
   write (IOunit, '(a,2x,i4,2x,a,i2,a)') 'tdef', ntimes,  &
        'linear 00:00z04jul1997  ', i, 'mn'
else
   i = nint(snap_tsep_bmin/60._f4)
   if (i <= 9) then
      write (IOunit, '(a,2x,i4,2x,a,i1,a)') 'tdef', ntimes,  &
           'linear 00:00z04jul1997  ', i, 'hr'
   else
      write (IOunit, '(a,2x,i4,2x,a,i2,a)') 'tdef', ntimes,  &
           'linear 00:00z04jul1997  ', i, 'hr'
   end if
end if
write (IOunit, '(a,2x,i2)') 'vars', nvar
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'temp', nk, 99, 'temperature [K]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'p', nk, 99, 'atm pressure [Pa]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'u', nk, 99, 'u-wind [m/s]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'v', nk, 99, 'v-wind [m/s]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'az', nk, 99, 'areopotential height [m]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Ov_mc', nk, 99,  &
     'H2O_vapor mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'dust_mc', nk, 99,  &
     'dust aer mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Oi_mc', nk, 99,  &
     'H2O_ice aer mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Od_mc', nk, 99,  &
     'H2Oi_dust aer mass conc [kg/m^3]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'gt_SBS', nk_SBS, 99,  &
     'subsfc temperature [K]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'gt_sfc', 0, 99,  &
     'sfc temp or ice mass [K or -kg]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'ghi_sfc', 0, 99,  &
     'sfc H2O ice mass [kg/m^2]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'dust_asfc', 0, 99,  &
     'sfc dust aer mass [kg/m^2]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Oi_asfc', 0, 99,  &
     'sfc H2O_ice aer mass [kg/m^2]'
write (IOunit, '(a,2x,i3,2x,i2,2x,a)') 'H2Od_asfc', 0, 99,  &
     'sfc H2Oi_dust aer mass [kg/m^2]'

write (IOunit, '(a/)') 'endvars'
close(IOunit)

call PIPPf_dealloc()
deallocate(t_m,u_m,v_m,u_a,v_a,p_a,sp_m,sp_a,gt_sfc_m,gi_sfc_m,gt_SBS_m,  &
     gpz_a,t_a,gt_sfc_a,sigma_levm,nzi,vt,gt_SBS_a,sol_Ls,lat0,lon0,rho_a,  &
     ghi_sfc_m,ghi_sfc_a,dustmr_m,H2Ovmr_m,H2Ovmc_a,  &
     dustmc_a,H2Oimc_a,H2Odmc_a,dust_asfc_m,  &
     dust_asfc_a,H2Oi_asfc_a,H2Od_asfc_a)
!deallocate(t_m,u_m,v_m,u_a,v_a,p_a,sp_m,sp_a,gt_sfc_m,gi_sfc_m,gt_SBS_m,  &
!     gpz_a,t_a,gt_sfc_a,sigma_levm,nzi,vt,gt_SBS_a,sol_Ls,lat0,lon0,rho_a,  &
!     ghi_sfc_m,ghi_sfc_a,H2Ovmr_m,dustmr_m,H2Oimr_m,H2Odmr_m,H2Ovmc_a,  &
!     dustmc_a,H2Oimc_a,H2Odmc_a,dust_asfc_m,H2Oi_asfc_m,H2Od_asfc_m,  &
!     dust_asfc_a,H2Oi_asfc_a,H2Od_asfc_a)

write (*, '(/,3(a,/))') 'Created the following file (sets): ',  &
     trim(gradsctlo), trim(outprfx)//'.*'

stop '** Normal completion **'


contains


!```````````````````````````````````````````````````````````````````
subroutine rdMGCM_UK_ncoutp(phase)

!+ Reads data from UK (Open Univ -- formerly Oxford) MGCM NetCDF output files

use univ_kind_defs_mod, only: f4
use netcdf, only: nf90_nowrite, nf90_open, nf90_close, nf90_get_var,  &
     nf90_inq_dimid, nf90_inquire_dimension, nf90_inq_varid, nf90_get_var,  &
     nf90_close

implicit none

!=== Argument declarations:

integer, intent(IN) :: phase

!== Local declarations:

real(f4), allocatable, dimension(:,:,:) :: dump3D      ! [ni,nj,nk]
real(f4), allocatable, dimension(:,:) :: dump2D           ! [ni,nj]
real(f4), allocatable, dimension(:) :: dump1D      ! [nk]
real(f4), allocatable, dimension(:) :: dump1D_y   ! [nj]
real(f4), allocatable, dimension(:,:,:) :: dump3Dsfc        ! [ni,nj,nk_SBS]
real(f4), allocatable, dimension(:) :: dump1Dsfc        ! [nk_SBS]

real(f4) :: time_tmp

integer :: ncid, itmp_0D, varid_tmp, isnap, ic, nc
integer, dimension(3) :: start_t2Ds
integer, dimension(4) :: start_t3Ds
character(len=128) :: fn_tmp

!=== Executable statements:

! Open MGCM output data file:
call nc_check(nf90_open(dfname_wp(1:len_trim(dfname_wp)), nf90_nowrite, ncid))

if(phase == 0) then      ! Read "header" info

   call nc_check(nf90_inq_dimid(ncid, 'lat', lat_dimid))
   call nc_check(nf90_inq_dimid(ncid, 'lon', lon_dimid))
   call nc_check(nf90_inq_dimid(ncid, 'sigma', levm_dimid))
!%   call nc_check(nf90_inq_dimid(ncid, 'sbs_lyr_mid', sbslevm_dimid))
   call nc_check(nf90_inq_dimid(ncid, 'time', rec_dimid))
   
   call nc_check(nf90_inquire_dimension(ncid, lat_dimid, len=itmp_0D))
   nj1 = itmp_0D+1; nj = itmp_0D; nj_1 = itmp_0D-1

   call nc_check(nf90_inquire_dimension(ncid, lon_dimid, len=itmp_0D))
   ni = itmp_0D 

   call nc_check(nf90_inquire_dimension(ncid, levm_dimid, len=itmp_0D))
   nk = itmp_0D

!%   call nc_check(nf90_inquire_dimension(ncid, sbslevm_dimid, len=itmp_0D))
!%   nk_SBS = itmp_0D
   nk_SBS = 11   ! Will "construct" subsurface soil temperature data

   ! Allocate work arrays:
   allocate(dump1D(nk), dump1Dsfc(nk_SBS),dump1D_y(nj))

!%   call nc_check(nf90_inq_varid(ncid, 'nsols_per_ncfile', varid_tmp))
!%   call nc_check(nf90_get_var(ncid, varid_tmp, nsolpf))
   nsolpf = 30   ! Calculated from example output file

!%   call nc_check(nf90_inq_varid(ncid, 'nsnaps_per_sol', varid_tmp))
!%   call nc_check(nf90_get_var(ncid, varid_tmp, ntpsl))
   ntpsl = 12   ! Calculated from example output file

!%   call nc_check(nf90_inq_varid(ncid, 'p_top', varid_tmp))
!%   call nc_check(nf90_get_var(ncid, varid_tmp, sptrop))
   sptrop = 0._f4     ! S. Lewis, personal communication

!%   call nc_check(nf90_inq_varid(ncid, 'g', varid_tmp))
!%   call nc_check(nf90_get_var(ncid, varid_tmp, g))
   g = 3.72_f4   ! Guessed from 'controle' data

!%   call nc_check(nf90_inq_varid(ncid, 'R_air', varid_tmp))
!%   call nc_check(nf90_get_var(ncid, varid_tmp, rgas))
   rgas = 191.2_f4   ! From S. Lewis; might be 191.182

!%   call nc_check(nf90_inq_varid(ncid, 'cp_air', varid_tmp))
!%   call nc_check(nf90_get_var(ncid, varid_tmp, cp))
   cp = rgas/0.233_f4   ! From S. Lewis; R/cp= 0.233

   allocate(sol_Ls(nsolpf+1), sigma_levm(nk), zlevSBS(nk_SBS+1), lat0(nj),  &
        lon0(ni))

   call nc_check(nf90_inq_varid(ncid, 'Ls', varid_tmp))
   call nc_check(nf90_get_var(ncid, varid_tmp, sol_Ls(1:nsolpf),  &
        start=(/1/), stride=(/ntpsl/)))
   it = nsolpf; sol_Ls(it+1) = sol_Ls(it)+(sol_Ls(it)-sol_Ls(it-1))

   call nc_check(nf90_inq_varid(ncid, 'lat', varid_tmp))
   call nc_check(nf90_get_var(ncid, varid_tmp, dump1D_y))
   do j=1,nj
      lat0(j) = dump1D_y(1+nj-j)    ! Reverse ordering
   end do

   call nc_check(nf90_inq_varid(ncid, 'lon', varid_tmp))
   call nc_check(nf90_get_var(ncid, varid_tmp, lon0))

   call nc_check(nf90_inq_varid(ncid, 'sigma', varid_tmp))
   call nc_check(nf90_get_var(ncid, varid_tmp, dump1D))
   do k=1,nk
!%      sigma_levm(1+nk-k) = dump1D(k)
      sigma_levm(k) = dump1D(k)   ! No order reversal needed
   enddo

!%   call nc_check(nf90_inq_varid(ncid, 'sbs_lyr_mid', varid_tmp))
!%   call nc_check(nf90_get_var(ncid, varid_tmp, dump1Dsfc))
!%   do k=1,nk_SBS
!%      zlevSBS(1+nk_SBS-k) = -dump1Dsfc(k)  ! Reverse_order/negate (PRAMS use)
!%   enddo
   zlevSBS(1:nk_SBS) = (/ -1.6_f4, -0.12_f4, -0.10_f4, -0.08_f4, -0.06_f4,  &
                          -0.04_f4, -0.02_f4, -0.01_f4, -0.005_f4, -0.002_f4,  &
                          0._f4 /)  ! Fake; similar to PRAMS default

   ! Deallocate work arrays:
   deallocate(dump1D, dump1Dsfc, dump1D_y)

elseif(phase == 1) then  ! Read "data"

   ! Allocate work arrays:
   allocate(dump3D(ni,nj,nk), dump3Dsfc(ni,nj,nk_SBS) ,dump2D(ni,nj))

   ! Read MGCM output data file:

!% 'topo' is now filled in separate NetCDF file read below
!%   call nc_check(nf90_inq_varid(ncid, 'topo', varid_tmp))
!%   call nc_check(nf90_get_var(ncid, varid_tmp, topo(:,:,1)))   ! [m]

!% 'utc_ts' is now filled in time-dependent loop below
!%   call nc_check(nf90_inq_varid(ncid, 'UT_time', varid_tmp))
!%   call nc_check(nf90_get_var(ncid, varid_tmp, utc_ts(:,1))) ! [Mars-hours UT]

   isnap = 0
   do n=1,nsolpf      ! Read each sol from the data file in sequence
      do it=1,ntpsl       ! Number of times per sol that MGCM output is made
         isnap = isnap+1

         utc_ts(isnap,1) = real(it-1, kind=f4)/real(ntpsl, kind=f4)*24._f4
 
         ! Report progress to user:
         write (*,'(a,i2,a,i2,a)') 'Reading SOL=',n,' TIMESTEP=',it,' ...'

         start_t2Ds = (/ 1, 1, isnap /)
         start_t3Ds = (/ 1, 1, 1, isnap /)

         call nc_check(nf90_inq_varid(ncid, 't', varid_tmp))
         call nc_check(nf90_get_var(ncid, varid_tmp, dump3D, start=start_t3Ds))
         do k=1,nk
            do j=1,nj
               t_m(:,j,k,it,n) = dump3D(:,1+nj-j,k)      ! [K]
            end do
         end do
        
         call nc_check(nf90_inq_varid(ncid, 'u', varid_tmp))
         call nc_check(nf90_get_var(ncid, varid_tmp, dump3D, start=start_t3Ds))
         do k=1,nk
            do j=1,nj
               u_m(:,j,k,it,n) = dump3D(:,1+nj-j,k)      ! [m/s]
            end do
         end do

         call nc_check(nf90_inq_varid(ncid, 'v', varid_tmp))
         call nc_check(nf90_get_var(ncid, varid_tmp, dump3D, start=start_t3Ds))
         do k=1,nk
            do j=1,nj
               v_m(:,j,k,it,n) = dump3D(:,1+nj-j,k)      ! [m/s]
            end do
         end do

!%         call nc_check(nf90_inq_varid(ncid, 'H2Ov', varid_tmp))
!%         call nc_check(nf90_get_var(ncid, varid_tmp, dump3D, start=start_t3Ds))
!%         do k=1,nk
!%            H2Ovmr_m(:,:,1+nk-k,it,n) = dump3D(:,:,k)      ! [kg/kg_air]
!%         enddo

!%         call nc_check(nf90_inq_varid(ncid, 'dust_mass', varid_tmp))
!%         call nc_check(nf90_get_var(ncid, varid_tmp, dump3D, start=start_t3Ds))
!%         do k=1,nk
!%            dustmr_m(:,:,1+nk-k,it,n) = dump3D(:,:,k)      ! [kg/kg_air]
!%         enddo

!%         call nc_check(nf90_inq_varid(ncid, 'H2Oi_mass', varid_tmp))
!%         call nc_check(nf90_get_var(ncid, varid_tmp, dump3D, start=start_t3Ds))
!%         do k=1,nk
!%            H2Oimr_m(:,:,1+nk-k,it,n) = dump3D(:,:,k)      ! [kg/kg_air]
!%         enddo

!%         call nc_check(nf90_inq_varid(ncid, 'H2Od_mass', varid_tmp))
!%         call nc_check(nf90_get_var(ncid, varid_tmp, dump3D, start=start_t3Ds))
!%         do k=1,nk
!%            H2Odmr_m(:,:,1+nk-k,it,n) = dump3D(:,:,k)      ! [kg/kg_air]
!%         enddo

         call nc_check(nf90_inq_varid(ncid, 'ps', varid_tmp))
         call nc_check(nf90_get_var(ncid, varid_tmp, dump2D, start=start_t2Ds))
         do j=1,nj
            sp_m(:,j,it,n) = dump2D(:,1+nj-j)      ! [Pa]
         end do

         call nc_check(nf90_inq_varid(ncid, 'tsurf', varid_tmp))
         call nc_check(nf90_get_var(ncid, varid_tmp, dump2D, start=start_t2Ds))
         do j=1,nj
            gt_sfc_m(:,j,it,n) = dump2D(:,1+nj-j)      ! [K]
         end do

         call nc_check(nf90_inq_varid(ncid, 'co2ice', varid_tmp))
         call nc_check(nf90_get_var(ncid, varid_tmp, dump2D, start=start_t2Ds))
         do j=1,nj
            gi_sfc_m(:,j,it,n) = dump2D(:,1+nj-j)      ! [kg/m^2]
         end do

!%         call nc_check(nf90_inq_varid(ncid, 'H2Oi_sfcmass', varid_tmp))
!%         call nc_check(nf90_get_var(ncid, varid_tmp, dump2D, start=start_t2Ds))
!%         ghi_sfc_m(:,:,it,n) = dump2D(:,:)      ! [kg/m^2]
         ghi_sfc_m(:,:,it,n) = 0._f4     ! [kg/m^2]

!%         call nc_check(nf90_inq_varid(ncid, 'dust_mass_s', varid_tmp))
!%         call nc_check(nf90_get_var(ncid, varid_tmp, dump2D, start=start_t2Ds))
!%         dust_asfc_m(:,:,it,n) = dump2D(:,:)      ! [kg/m^2]

!%         call nc_check(nf90_inq_varid(ncid, 'H2Oi_mass_s', varid_tmp))
!%         call nc_check(nf90_get_var(ncid, varid_tmp, dump2D, start=start_t2Ds))
!%         H2Oi_asfc_m(:,:,it,n) = dump2D(:,:)      ! [kg/m^2]

!%         call nc_check(nf90_inq_varid(ncid, 'H2Od_mass_s', varid_tmp))
!%         call nc_check(nf90_get_var(ncid, varid_tmp, dump2D, start=start_t2Ds))
!%         H2Od_asfc_m(:,:,it,n) = dump2D(:,:)      ! [kg/m^2]

!%         call nc_check(nf90_inq_varid(ncid, 'T_sbs', varid_tmp))
!%         call nc_check(nf90_get_var(ncid, varid_tmp, dump3Dsfc,  &
!%              start=start_t3Ds))
!%         do k=1,nk_SBS
!%            gt_SBS_m(:,:,1+nk_SBS-k,it,n) = dump3Dsfc(:,:,k)      ! [K]
!%         enddo
         do j=1,nj
            do i=1,ni
               time_tmp = utc_ts(isnap,1)+lon0(i)/15._f4    ! LMST
               if (time_tmp < 0._f4) time_tmp = time_tmp+24._f4
               time_tmp = mod(time_tmp, 24._f4)
               gt_SBS_m(i,j,1:nk_SBS,it,n) = SBS_temp(nk_SBS,  &
                    gt_sfc_m(i,j,it,n), time_tmp)
            end do
         end do

      end do
   end do
   snap_tsep_bmin = max(utc_ts(2,1)-utc_ts(1,1), utc_ts(2,1)-utc_ts(1,1))*  &
        60._f4  ! [body-minutes]

   call nc_check(nf90_close(ncid))       ! Close MGCM output data file

!=====
   ! Open MGCM topo data file (do not need to reorder latitude):
   nc = len_trim(dfname_wp)
   ic = index(dfname_wp(1:nc),'/',.TRUE.) ! Find last forward slash in filename
   if (ic /= 0) then
      fn_tmp = dfname_wp(1:ic)//'mola72x36.nc'
   else
      fn_tmp = './mola72x36.nc'
   end if
   call nc_check(nf90_open(fn_tmp, nf90_nowrite, ncid))
   call nc_check(nf90_inq_varid(ncid, 'topogy', varid_tmp))
   call nc_check(nf90_get_var(ncid, varid_tmp, topo(:,:,1)))   ! [m]
   call nc_check(nf90_close(ncid))       ! Close MGCM topo data file
!=====

   ! Deallocate work arrays:
   deallocate(dump3D, dump3Dsfc, dump2D)

   !**** Temporarily fill water stuff here: ****
         !UNFINISHED
      
end if

end subroutine rdMGCM_UK_ncoutp


!```````````````````````````````````````````````````````````````````
subroutine p_gpz_calc

!+ Calculates vertical pressure levels and areopotential heights

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

integer :: i,j,k
real(f4) :: H,tbot,pbot,gzsum,tavg

!=== Executable statements:
     
! Calculate MGCM vertical pressure field from surface pressure and sigma 
!   levels:

do i=1,ni
   do j=1,nj

      ! Construct MGCM vertical pressure profiles:

      do k=1,nk
         p_a(i,j,k) = sigma_levm(k)*(sp_a(i,j)-sptrop)+sptrop   ! [Pa]
      enddo

      ! Compute geopotential near surface from surface pressure

      gzsum = 0._f4
      pbot = sp_a(i,j)  ! Begin integration at surface pressure
      tbot = t_a(i,j,1)   ! Not at all the best value for this, but oh well
      do k=1,nk   ! Loop over all MGCM layers 
         tavg = (tbot+t_a(i,j,k))*0.5_f4 ! Average tempk for the current layer
         H = (-rgas/g)*tavg*log(p_a(i,j,k)/pbot)  ! Delta-geopotential height
         gzsum = gzsum+H       ! Geopotential height of current layer *AGL*
         gpz_a(i,j,k) = gzsum  !   (Ignore whatever gpz MGCM has)
         pbot = p_a(i,j,k)   ! These will become the lower boundary for the
         tbot = t_a(i,j,k)   !   next iteration
      enddo

   enddo
enddo

end subroutine p_gpz_calc


!```````````````````````````````````````````````````````````````````
subroutine nc_check(status)

!+ Checks NetCDF procedure status
  
use netcdf, only: nf90_noerr, nf90_strerror

implicit none

!=== Argument declarations:

integer, intent(IN) :: status

!== Local declarations:

! NONE

!=== Executable statements:

if(status /= nf90_noerr) then 
   write (*,'(/a/)') trim(nf90_strerror(status))
   stop
endif

end subroutine nc_check


!```````````````````````````````````````````````````````````````````
function SBS_temp(nk_SBS, gt_sfc, LMST) result(gt_SBS)

!+ Generate sub-surface temperature profiles based on time of sol, etc.

use univ_kind_defs_mod, only: f4

implicit none

!=== Argument declarations:

integer, intent(IN) :: nk_SBS
real(f4), intent(IN) :: gt_sfc,  &    ! Surface temperature [K]
                    LMST    ! Local mean solar time [decimal Mars-hours]

real(f4), dimension(nk_SBS) :: gt_SBS    ! Function result declaration

!== Local declarations:

real(f4), dimension(11,4) :: gtadj      ! *** HARDWIRED ***

integer :: tidx, tidx1, iq
real(f4) :: adjtd, tint

! Arbitrary temp differences between SBS levels (from an early Gusev run):

data gtadj(11:1:-1,1) / -2.144_f4, -4.017_f4, -6.617_f4, -9.306_f4,  &
     -2.838_f4, 1.412_f4, 1.480_f4, 0.0383_f4, -1.262_f4, -1.937_f4,  &
     0._f4 /     ! Midnight
data gtadj(11:1:-1,2) / -1.460_f4, -2.815_f4, -4.910_f4, -8.160_f4,  &
     -6.225_f4, -3.100_f4, -1.950_f4, -2.140_f4, -2.778_f4, -3.321_f4,  &
     0._f4 /     ! 6am
data gtadj(11:1:-1,3) / 6.545_f4, 12.101_f4, 19.109_f4, 23.410_f4, 6.167_f4,  &
     -1.548_f4, -2.780_f4, -2.752_f4, -2.775_f4, -2.867_f4, 0._f4 /     ! Noon
data gtadj(11:1:-1,4) / -3.950_f4, -5.467_f4, -4.165_f4, 6.062_f4, 12.559_f4,  &
     6.301_f4, 1.094_f4, -1.458_f4, -2.416_f4, -2.701_f4, 0._f4 /        ! 6pm

!=== Executable statements:

adjtd = 24._f4/4._f4       ! LMST interval between representative profiles

gt_SBS(1) = 170._f4      ! "Deep" regolith temperature

tidx = int(LMST/adjtd)+1
if(tidx <= 3) then
   tidx1 = tidx+1
else
   tidx1 = 1
end if
tint = (LMST/adjtd)-real(tidx-1, kind=f4)

gt_SBS(nk_SBS) = gt_sfc
do iq=nk_SBS-1,2,-1
   gt_SBS(iq) = gt_SBS(iq+1)-((1.-tint)*gtadj(iq+1,tidx)+  &
        tint*gtadj(iq+1,tidx1))    ! Interpolate between representative
                                   !   SBS temperature profiles
   gt_SBS(iq) = max(gt_SBS(iq), 140._f4)   ! A mostly arbitrary cutoff of 140K
end do

end function SBS_temp


!```````````````````````````````````````````````````````````````````
subroutine usage

!+ Prints program usage information

use univ_exception_mod, only: exception, FATAL

implicit none

!=== Argument declarations:

! NONE

!== Local declarations:

! NONE

!=== Executable statements:

write (*,'(/a/)')  &
     'USAGE:  Must be started from run_prep, or a similar helper script'
call exception('usage', FATAL)

end subroutine usage


end program split_UK_ncoutp
